import io
import json
import os
import pkg_resources


class Diccionario(object):
    def __init__(símismo):
        símismo.direc = pkg_resources.resource_filename('tinamit.Interfaz', 'Trads.json')
        with open(símismo.direc, encoding='utf8') as d:
            símismo.dic = json.load(d)

        símismo.lenguas = símismo.dic['Lenguas']
        símismo.estándar = 'Español'

        símismo.verificar_estados()

        símismo.leng_act = símismo.lenguas[símismo.dic['Actual']]
        símismo.trads_act = símismo.leng_act['Trads']
        símismo.izq_a_derech = símismo.leng_act['IzqaDerech']

    def guardar(símismo):
        with io.open(símismo.direc, 'w', encoding='utf8') as d:
            json.dump(símismo.dic, d, ensure_ascii=False, sort_keys=True, indent=2)

    def verificar_estados(símismo):
        estándar = símismo.lenguas[símismo.estándar]

        for nombre, leng in símismo.lenguas.items():
            llenos = []
            for frase in estándar['Trads']:
                if frase in leng['Trads'].keys() and leng['Trads'][frase] != '':
                    llenos.append(1)
                else:
                    leng['Trads'][frase] = ''
                    llenos.append(0)
            símismo.lenguas[nombre]['Estado'] = sum(llenos)/len(llenos)

        for nombre, leng in símismo.lenguas.items():
            for frase in leng['Trads'].copy():
                if frase not in estándar['Trads'].keys():
                    leng['Trads'].pop(frase)



import json
import os
import tkinter as tk

# Una función para modificar los formatos según la dirección del texto de la lengua
direc = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'Trads.json')
with open(direc, encoding='utf8') as d:
    dic = json.load(d)
leng = dic['Actual']
IzqaDerech = dic['Lenguas'][leng]['IzqaDerech']


def gen_formato(formato):
    if IzqaDerech:
        return formato
    else:
        if type(formato) is str:
            invers = [('e', 'w'), ('ne', 'nw'), ('se', 'sw'), ('right', 'left'), ('izquierda', 'derecha')]
            for inver in invers:
                if formato == inver[0]:
                    formato = inver[1]
                    break
                elif formato == inver[1]:
                    formato = inver[0]

        if type(formato) is dict:
            formato = formato.copy()
            invers = [('e', 'w'), ('ne', 'nw'), ('se', 'sw'), ('right', 'left')]
            if 'x' in formato.keys():
                if 'relx' not in formato.keys():
                    formato['relx'] = 0
            if 'x' in formato.keys() or 'relx' in formato.keys():
                if 'anchor' not in formato.keys():
                    formato['anchor'] = tk.NW

            for ll, v in formato.items():
                if ll == 'x':
                    formato[ll] = -v
                if ll == 'relx':
                    formato[ll] = (1 - v)
                    continue
                for inver in invers:
                    if v == inver[0]:
                        formato[ll] = inver[1]
                        continue
                    elif v == inver[1]:
                        formato[ll] = inver[0]

        return formato


# Parámetros generales
col_fondo = 'white'  # El color de fondo del interfaz
color_bts = {'1': '#%02x%02x%02x' % (218, 242, 142),
             '2': '#%02x%02x%02x' % (181, 230, 29),
             '3': '#%02x%02x%02x' % (102, 204, 0),
             '4': '#%02x%02x%02x' % (128, 64, 0)}
color_gris = '#C3C3C3'
col_1 = color_bts['3']
col_2 = '#cc9900'
col_3 = color_bts['2']
col_4 = color_bts['1']
col_5 = '#33cc33'
fuente = 'Comic Sans MS'
formato_cajas = dict(bg=col_fondo, borderwidth=0, highlightthickness=0)
formato_botones = dict(borderwidth=0, highlightthickness=0)
formato_etiq = dict(bg=col_fondo)

formato_bts_norm = dict(activebackground=col_1, bg=col_3, fg='#000000', **formato_botones)
formato_bts_sel = dict(bg=col_1)

# Ventana central
ancho_ventana = 1100
altura_ventana = 800


# Caja inicial
formato_CjInic = dict(bg=col_fondo)
ubic_CjInic = dict(x=0, y=0, relwidth=1, relheight=1)
formato_LogoInic = dict(borderwidth=0, highlightthickness=0)
ubic_LogoInic = dict(side='top', pady=(175, 50))
formato_BtsInic = dict(font=(fuente, 25, 'bold'), height=2, width=13, **formato_bts_norm)

ubic_BtsInic = dict(side='left', ipadx=5, ipady=5, padx=10, pady=10)


# Caja lenguas
ubic_CjLeng = dict(relx=0, y=0, relwidth=1, relheight=1)
ubic_BtRegrCent = dict(x=20, y=20)
ubic_CbzLeng = dict(relx=0.5, y=20, anchor=tk.N)
formato_CbzLeng = dict(font=(fuente, 40, 'bold'), fg=col_1, **formato_etiq)

ubic_CjCentLeng = dict(relx=0.5, rely=0, x=0, y=100, relheight=1, height=-150, relwidth=1, anchor=tk.N)
formato_LínVert = dict(bd=0, highlightthickness=0, bg=col_5)
ubic_LínVert1 = dict(relx=1/3, rely=0.05, y=20, width=1, relheight=1, anchor=tk.N)
ubic_LínVert2 = dict(relx=2/3, rely=0.05, y=20, width=1, relheight=1, anchor=tk.N)

ubic_CjIzqLeng = dict(relx=(1/3)/2, rely=0.05, y=20, relwidth=1/3, width=-10, relheight=1, anchor=tk.N)
ubic_CjMedLeng = dict(relx=1/3 + (1/3)/2, rely=0.05, y=20, relwidth=1/3, width=-10, relheight=1, anchor=tk.N)
ubic_CjDerchLeng = dict(relx=2/3 + (1/3)/2, rely=0.05, y=20, relwidth=1/3, width=-10, relheight=1, anchor=tk.N)

formato_EtiqLengCentro = dict(font=(fuente, 30, 'bold'), fg=col_5, **formato_etiq)
formato_EtiqLengLados = dict(font=(fuente, 20, 'bold'), fg=col_3, **formato_etiq)
ubic_EtiqCbzColsLeng = dict(relx=0.5, rely=0, y=50, anchor=tk.S)

formato_CjLstLengCentro = dict(highlightthickness=1, highlightbackground=col_5, bg=col_fondo)
formato_CjLstLengLados = dict(highlightthickness=1, highlightbackground=col_3, bg=col_fondo)
ubic_LstsLeng = dict(relx=0.5, rely=0, y=75, width=300, relheight=1, height=-150, anchor=tk.N)
ubic_LstsLeng_bajo = dict(relx=0.5, rely=0, y=75+20, width=300, relheight=1, height=-150-20, anchor=tk.N)
ubic_CjAñadirLeng = dict(relx=0, rely=0, x=20, y=60, height=20, anchor=tk.NW)
ubic_CjCamposAñLeng = dict(side='left', padx=2)
formato_cj_etiq_nombre_leng = dict(height=25, **formato_cajas)
formato_etiq_nombre_leng = dict(anchor=tk.W, **formato_etiq)
ubic_etiq_nombre_leng = dict(side='left')
ancho_etiq_nombre_largo = 160
ancho_etiq_nombre_estr = 80
ubic_IzqLstLeng = dict(side='left', padx=5)
ubic_EtiqNombreLstLeng = dict(side='left')
ubic_DerLstLeng = dict(side='right', padx=5)
ubic_CjCentLstLeng = dict(side='left', padx=0)
colores_prog_leng = [(152, 103, 52), (181, 230, 29)]
dim_barra_prog_leng = (20, 55)
ubic_BtsAñadirLeng = dict(side='right', padx=2)
ubic_CentralItemaLstLeng = dict(side='left')

formato_CjEditLeng = dict(highlightthickness=1, highlightbackground=col_5, bg=col_fondo)
ubic_CjEditLeng = dict(relx=0.5, rely=0, y=10, relheight=1, width=600, height=-20, anchor=tk.N)
formato_EtiqCbzEditLeng = dict(font=(fuente, 40, 'bold'), fg=col_2, **formato_etiq)
ubic_CjCbzCjEditLeng = dict(side='top', pady=0, fill=tk.X)
ubic_CjNomEditLeng = dict(side='top', pady=0, fill=tk.X)
formato_EtiqLengBase = dict(font=(fuente, 20, 'bold'), fg=col_5, **formato_etiq)
formato_EtiqLengEdit = dict(font=(fuente, 20, 'bold'), fg=col_3, **formato_etiq)
ubic_EtiqLengs = dict(side='left', padx=5, pady=5, fill=tk.BOTH, expand=True)

formato_LínHor = dict(bd=0, highlightthickness=0, bg=col_3, height=1)
ubic_LínHorCjEditLeng = dict(side='top', fill=tk.X)
ubic_CjCentrCjEditLeng = dict(side='top', fill=tk.BOTH, expand=True)

formato_LstEditLeng = dict(highlightthickness=0, highlightbackground=col_3, bg=col_fondo)
ubic_LstEditLeng = dict(relx=0, rely=0, relheight=1, width=ubic_CjEditLeng['width'], anchor=tk.NW)

ubic_CjBtsCjEditLeng = dict(side='top', pady=20, fill=tk.X)
formato_CjLengTxOrig = dict(bg=col_fondo, highlightthickness=1, highlightbackground=col_5)
formato_EtiqLengTxOrig = dict(font=(fuente, '14', 'bold'), fg=col_5, bg=col_fondo,
                              width=20, wraplength=240, anchor=tk.W, justify=tk.LEFT)
ubic_CjTxOrigTradLeng = dict(side='left', padx=5, pady=5, fill=tk.Y, expand=True)
formato_CampoTexto = dict(font=(fuente, '14', 'bold'), bg=col_fondo,
                          highlightthickness=1, highlightbackground=col_3, highlightcolor=col_5,
                          width=25, height=3, wrap=tk.WORD, relief=tk.FLAT)
ubic_CamposLeng = dict(side='left', padx=25, pady=25, expand=True)


formato_CajaAvisoReinic = dict(highlightthickness=3, highlightbackground=col_2, bg=col_fondo)
ubic_CjAvisoReinic = dict(relx=0.5, rely=0.5, width=350, height=200, anchor=tk.CENTER)
formato_EtiqLengReinic = dict(font=(fuente, 14), fg=col_2, wraplength=275, **formato_etiq)
ubic_etiq_aviso_inic_leng = dict(side='top', pady=(20, 0))
formato_BtAvisoInic = dict(borderwidth=0, highlightthickness=0, activebackground='#ffdb4d',
                           font=(fuente, 20, 'bold'), height=1, width=9,  wraplength=170)  # Formatos generales
formato_BtAvisoInic_norm = dict(bg='#ffe580', fg='#000000', **formato_BtAvisoInic)
formato_BtAvisoInic_sel = dict(bg='#ffdb4d')
ubic_bt_aviso_inic_leng = dict(side='bottom', pady=(0, 20))

formato_EtiqLengBorrar = dict(font=(fuente, 14), fg=col_2, wraplength=275, **formato_etiq)
ubic_etiq_aviso_borrar_leng = dict(side='top', pady=(20, 0))

ubic_bts_aviso_borrar_leng = dict(side='left', padx=25)
ubic_cj_bts_aviso_borrar_leng = dict(side='bottom', pady=(0, 20))


# Caja central
formato_CjCent = dict(bg=col_fondo)
ubic_CjCent = dict(x=0, y=0, relwidth=1, relheight=1)


# Caja cabeza
formato_CjCabeza = dict(height=110, **formato_cajas)
ubic_CjCabeza = dict(side='top', fill=tk.X, anchor=tk.N)

ubic_BtsArchivo = dict(x=20, rely=0.5, anchor=tk.W)
ubic_BtNuevo = dict(row=0, column=0, padx=5, pady=5)
ubic_BtGuardar = dict(row=0, column=1, padx=5, pady=5)
ubic_BtAbrir = dict(row=1, column=0, padx=5, pady=5)
ubic_BtGuardarComo = dict(row=1, column=1, padx=5, pady=5)

formato_LogoCabz = dict(borderwidth=0, highlightthickness=0, height=65)
ubic_LogoCabz = dict(relx=0.5, rely=0.5, anchor=tk.CENTER)

formato_BtLeng = dict(height=40, width=40, **formato_botones)
ubic_BtLeng = dict(relx=1, rely=0.5, x=-20, anchor=tk.E)

# Botones izquierda
ubic_CjIzq = dict(side='top', fill=tk.Y, expand=True, anchor=tk.W)
formato_BtsNavIzq = dict(width=86, height=86, borderwidth=0, highlightthickness=0)
ubic_CjBtNavIzq = dict(side='top', padx=10, pady=15, expand=True)
ubic_BtNavIzq = dict(side='left', padx=10)
formato_lín_bts = dict(highlightthickness=0, width=7, height=120)
ubic_LínNavIzq = dict(side='right', padx=10)
formato_LínIzq = dict(bd=0, highlightthickness=0, width=2, bg='black')
ubic_LínIzq = dict(side='right', pady=(0, 20), fill=tk.Y, expand=True)

# Caja Contenedora de cajas etapas
ubic_CjContCjEtps = dict(side='right', fill=tk.BOTH, expand=True, anchor=tk.NW)

# Cajas etapas
ubic_CjEtp = dict(relx=0, rely=0, relwidth=1, relheight=1)
ubic_BtNavEtp_adel = dict(rely=1, relx=0.52, anchor=tk.S)
ubic_BtNavEtp_atrs = dict(rely=0, relx=0.52, anchor=tk.N)
formato_BtsNavEtapa = dict(width=270, height=35, bg=col_fondo, borderwidth=0, highlightthickness=0)
formato_EncbzCjEtp = dict(font=(fuente, 35, 'bold'), fg=col_1, **formato_etiq)
ubic_EncbzCjEtp = dict(relx=0, rely=0, x=20)


# Cajas sub etapas
ubic_CjSubEtp = dict(relx=0, rely=0, y=2*formato_BtsNavEtapa['height'],
                     relwidth=1, relheight=1,
                     height=-3*formato_BtsNavEtapa['height'])
formato_EncbzCjSubEtp = dict(font=(fuente, 20, 'bold'), fg=col_2, **formato_etiq)
ubic_EncbzCjSubEtp = dict(relx=0, rely=0, x=30)

ubic_BtNavSub_adel = dict(relx=1, rely=0.5, anchor=tk.E)
ubic_BtNavSub_atrs = dict(relx=0, rely=0.5, anchor=tk.W)
formato_BtsNavSub = dict(width=25, height=205, bg=col_fondo, borderwidth=0, highlightthickness=0)
formato_etiq_error = dict(bg='#ffffe5', fg='#ff0000', font=(fuente, '12'), wraplength=350, padx=10, pady=10)
formato_etiq_todobien = dict(bg='#f2ffcc', fg='#2eb82e', font=(fuente, '12'), wraplength=350, padx=10, pady=10)

# Listas
formato_CjLstItemas = dict(highlightthickness=1, highlightbackground=col_1, bg=col_fondo)
formato_TlLstItemas = dict(bg=col_fondo, highlightthickness=0)
formato_EtiqEncbzLst = dict(bg=col_4, fg=col_1, font=(fuente, 14, 'bold'), anchor=tk.W)
ubic_ColsEncbzLst = dict(y=0, relheight=1, anchor=tk.NW)
ubic_EncbzLstItemas = dict(x=0, y=0, relx=0, rely=0, height=25, relwidth=1, anchor=tk.NW)
ubic_TlLstItemas = dict(x=0, y=0, relx=0, rely=0, relheight=1, relwidth=1, anchor=tk.NW)
ubic_BaraDesp = dict(side="right", fill="y")
ubic_CjTl = dict(anchor=tk.NW)

formato_secciones_itemas = dict(height=20, **formato_cajas)
ubic_CjItemas = dict(side='top', pady=2, fill=tk.X, expand=True)
ancho_cj_bts_itemas = 85
ubic_CjColsItemas = dict(side='left', fill=tk.BOTH, expand=True)
ubic_BtsItemas = dict(side='left', padx=5)
ubic_CjBtsItemas = dict(side='right')
ubic_EtiqItemas = dict(side='left', anchor=tk.W)
fuente_etiq_itema_sel = (fuente, 14, 'bold')
fuente_etiq_itema_norm = (fuente, 14)
formato_texto_itemas = dict(font=(fuente, 14), fg='#000000', **formato_etiq)
ubic_ColsItemasEdit = dict(y=0, relheight=1, anchor=tk.NW)


# Controles
formato_EtiqCtrl = dict(bg=col_fondo, fg=col_3, font=(fuente, 20, 'bold'))
formato_EtiqCtrl_bloq = dict(fg='#999999')

formato_MnMn = dict(bg=col_fondo, relief='flat', bd=0, activebackground=col_4, activeforeground='#000000',
                    disabledforeground='#cccccc', font=(fuente, '13'))
formato_BtMn = dict(bg=col_fondo, highlightthickness=1, highlightbackground=col_3, highlightcolor=col_3,
                    relief='flat', activebackground=col_4, font=(fuente, '13'),
                    anchor=tk.W)
formato_BtMn_bloq = dict(highlightbackground='#999999')

ubic_EtiqMenú = dict(side='left', padx=20)
ubic_Menú = dict(side='left')

ubic_EtiqIngrNúm = dict(side='left', padx=20)
ubic_IngrNúm = dict(side='left')

formato_lín_escl = dict(fill=col_1, outline=col_1, disabledfill='#cccccc', disabledoutline='#cccccc')
ubic_Escl = dict(side='left')
ubic_CampoIngrEscl = dict(side='left', padx=10)
formato_EtiqNúmEscl = dict(font={fuente, 6}, fg=col_1, **formato_etiq)
ubic_EtiqNúmEscl = dict(side='left')
ubic_EtiqEscl = dict(side='left', padx=20)


formato_CampoIngr = dict(bg=col_fondo, highlightthickness=1, highlightbackground=col_3,
                         relief='flat', highlightcolor=col_3, font=(fuente, '13'))
formato_CampoIngr_error = dict(highlightthickness=2, highlightbackground='#ff0000')

formato_BtGuardar_norm = dict(bg='#6ac86a', fg='#ffffff', borderwidth=0, highlightthickness=0,
                              activebackground='#3ea73e', activeforeground='#ffffff',
                              font=(fuente, 15, 'bold'), width=10)
formato_BtGuardar_sel = dict(bg='#3ea73e', fg='#ffffff')
formato_BtGuardars_bloq = dict(bg='#ecf8ec', disabledforeground='#ffffff')
formato_BtBorrar_norm = dict(bg='#ff6233', fg='#ffffff', borderwidth=0, highlightthickness=0,
                             activebackground='#e63500', activeforeground='#ffffff',
                             font=(fuente, 15, 'bold'), width=10)
formato_BtBorrar_sel = dict(bg='#e63500', fg='#ffffff')
formato_BtBorrars_bloq = dict(bg='#ffebe5', disabledforeground='#ffffff')
ubic_BtsGrupo = dict(side='left', expand=True)
ubic_CjBtsGrupoCtrl = dict(side='top', padx=20, fill=tk.X, expand=True)


# Caja subetapa 1.1

ubic_caja_cargar_mds = dict(x=0, rely=0.2, relwidth=0.5, width=-10, height=250)
ubic_caja_cargar_bf = dict(relx=0.5, rely=0.2, x=+10, relwidth=0.5, width=-10, height=250)

formato_bts_cargar = dict(font=(fuente, 20, 'bold'), height=3, width=15, wraplength=300, **formato_bts_norm)
ubic_bts_cargar_mod = dict(relx=0.5, rely=0.3, y=10, anchor=tk.CENTER)
ubic_EtiqCargarMod = dict(side='bottom', pady=20)


# Caja subetapa 2.1
ubic_CjLstConecciones = dict(relx=0.5, rely=0, y=50, width=0.8 * ancho_ventana, height=150, anchor=tk.N)
anchos_cols_listacon = [0.40, 0.40, 0.20]
formato_etiq_conversión = dict(width=7, anchor=tk.W, **formato_texto_itemas)

ubic_CjBajoConex = dict(relx=0, rely=0, x=10, y=10, relwidth=1, relheight=.8, width=-20, anchor=tk.NW)

ubic_CjCtrlsConectar = dict(relx=0.5, rely=0, y=200, relwidth=1, width=-20, height=150, anchor=tk.N)
ancho_MnVars = 10
formato_EtiqsUnid = dict(font=(fuente, 13, 'italic'), fg='#d9b38c', bg=col_fondo)
ubic_EtiqsUnid = dict(side='top', anchor=tk.E)
ubic_CjsCtrlsConectar = dict(side='left', padx=20, pady=20, expand=True, anchor=tk.N)
ubic_CjMedCtrlsConectar = dict(side='left', padx=20, pady=30, expand=True, anchor=tk.N)
ubic_CtrlsConectar = dict(side='top')

ubic_CjBtsConectar = dict(relx=0.5, rely=0, y=400, width=400, anchor=tk.N)


# Caja subetapa 3.1
formato_etiqs_ctrls_tiempo = dict(font=(fuente, '13'), fg='#000000', bg=col_fondo)
ubic_CjCtrlsTiempo = dict(relx=0.5, rely=0.2, width=600, anchor=tk.CENTER)
ubic_CtrlsUnidTiempo = dict(side='left', padx=5)
ubic_CtrlsTiempo = dict(side='top', pady=10)

formato_bt_simular = dict(font=(fuente, 20, 'bold'), height=3, width=15, wraplength=300, **formato_bts_norm)
ubic_bt_simular = dict(side='top')
ubic_cj_bt_simular = dict(relx=0.5, rely=0.4, anchor=tk.N)
ubic_cj_corrida = dict(relx=0.5, rely=0.8, anchor=tk.N)
formato_CjSimulando = dict(height=50, width=350, **formato_cajas)
ubic_CjSimulando = dict(side='top', pady=20)
formato_etiq_simulando = dict(width=20, height=2, font=(fuente, 15, 'bold'), **formato_etiq)
ubic_EtiqSimulando = dict(relx=0, rely=0.5, anchor=tk.W)



import os
import sys
from importlib import import_module as importar_mod

from tinamit.Modelo import Modelo


class EnvolturaBF(Modelo):
    """
    Esta clase ofrece una envoltura para **TODOS** tipos de modelos biofísicos.

    Modelos biofísicos específicos se implementan por crear una subclase de `~tinamit.BF.ClaseModeloBF` específica
    para ellos.
    """

    def __init__(símismo, archivo):
        """
        Incializar la envoltura.

        :param archivo: El archivo con el
        :type archivo: str

        """

        dir_mod, nombre_mod = os.path.split(archivo)
        sys.path.append(dir_mod)

        módulo = importar_mod(os.path.splitext(nombre_mod)[0])

        try:
            modelo = módulo.Modelo
        except AttributeError:
            raise AttributeError('El archivo especificado ({}) no contiene una clase llamada Modelo.'.format(archivo))

        if callable(modelo):
            símismo.modelo = modelo()
        else:
            símismo.modelo = modelo

        if not isinstance(símismo.modelo, ClaseModeloBF):
            raise AttributeError('El archivo especificado ({}) contiene una clase llamada Modelo, pero'
                                 'esta clase no es una subclase de ClaseModeloBF.'.format(archivo))

        super().__init__(nombre='bf')

        símismo.vars_entrando = símismo.modelo.vars_entrando
        símismo.vars_saliendo = símismo.modelo.vars_saliendo

    def obt_unidad_tiempo(símismo):
        """

        :return:
        :rtype: str

        """

        return símismo.modelo.unidad_tiempo

    def inic_vars(símismo):
        """
        Inicializa los variables del modelo biofísico y los conecta al diccionario de variables del modelo.

        """

        # Crear el vínculo
        símismo.modelo.variables = símismo.variables

        # Inicializar los variables del modelo externo.
        símismo.modelo.inic_vars()

    def cambiar_vals_modelo_interno(símismo, valores):
        """

        :param valores:
        :type valores:

        """

        símismo.modelo.cambiar_vals(valores=valores)

    def incrementar(símismo, paso):
        """

        :param paso:
        :type paso:

        """

        símismo.modelo.incrementar(paso=paso)

    def leer_vals(símismo):
        """

        """
        símismo.modelo.leer_vals()

    def iniciar_modelo(símismo, **kwargs):
        """

        :param kwargs:
        :type kwargs:

        """

        símismo.modelo.iniciar_modelo(**kwargs)

    def cerrar_modelo(símismo):
        """

        """

        símismo.modelo.cerrar_modelo()


class ClaseModeloBF(Modelo):
    """
    Se debe desarrollar una subclase de esta clase para cada tipo modelo biofísico que se quiere volver compatible
    con Tinamit.
    """

    def __init__(símismo):
        """
        Esta función correrá automáticamente con la inclusión de `super().__init__()` en la función `__init__()` de las
        subclases de esta clase.
        """
        super().__init__(nombre='modeloBF')

    def cambiar_vals_modelo_interno(símismo, valores):
        """

        :param valores:
        :type valores:
        """
        raise NotImplementedError

    def incrementar(símismo, paso):
        """

        :param paso:
        :type paso:

        """
        raise NotImplementedError

    def leer_vals(símismo):
        """
        Esta función debe leer los variables del modelo desde el modelo externo y copiarlos al diccionario interno
        de variables. Asegúrese que esté *actualizando* el diccionario interno, y que no lo esté recreando, lo cual
        quebrará las conexiones con el modelo conectado.
        """
        raise NotImplementedError

    def iniciar_modelo(símismo, **kwargs):
        """

        :param kwargs:
        :type kwargs:
        """

        raise NotImplementedError

    def cerrar_modelo(símismo):
        """

        """
        raise NotImplementedError

    def obt_unidad_tiempo(símismo):
        """

        :return:
        :rtype: str
        """
        raise NotImplementedError

    def inic_vars(símismo):
        """
        MUY IMPORTANTE: Esta función debe modificar el diccionario que ya existe para símismo.variables, no crear un
        diccionario nuevo.
        Por ejemplo, NO HAGAS:
        |  símismo.variables = {var1: {...}, var2: {...}, ...}

        sino:
        |  símismo.variables[var1] = {...}
        |  símismo.variables[var2] = {...}

        Al no hacer esto, romperás la conección entre los diccionarios de variables de ClaseModeloBF y EnvolturaBF,
        lo cual impedirá después la conexión de estos variables con el modelo DS.

        """
        raise NotImplementedError



import math as mat
import time

from tinamit.Interfaz.Formatos import gen_formato as gf


def intercambiar(actual, nuevo, dirección):
    dirección = gf(dirección)
    x_act = actual.winfo_x()
    y_act = actual.winfo_y()
    ancho_act = actual.winfo_width()
    altura_act = actual.winfo_height()

    if dirección == 'izquierda':
        nuevo.place(x=(x_act + ancho_act), y=y_act)
        pos_inic = [x_act, x_act + ancho_act]
        distancia = ancho_act
    elif dirección == 'derecha':
        nuevo.place(x=(x_act - ancho_act), y=y_act)
        pos_inic = [x_act, x_act - ancho_act]
        distancia = ancho_act
    elif dirección == 'arriba':
        nuevo.place(x=x_act, y=(y_act + altura_act))
        pos_inic = [y_act, y_act + altura_act]
        distancia = altura_act
    elif dirección == 'abajo':
        nuevo.place(x=x_act, y=(y_act - altura_act))
        pos_inic = [y_act, y_act - altura_act]
        distancia = altura_act
    else:
        raise ValueError

    nuevo.lift()
    deslizar([actual, nuevo], pos_inic, dirección, distancia, paso=0.025, tiempo=0.5)


def sobreponer(actual, nuevo, dirección):
    dirección = gf(dirección)
    x_act = actual.winfo_x()
    y_act = actual.winfo_y()
    ancho_act = actual.winfo_width()
    altura_act = actual.winfo_height()

    if dirección == 'izquierda':
        nuevo.place(x=(x_act + ancho_act), y=y_act)
        pos_inic = [x_act + ancho_act]
        distancia = ancho_act
    elif dirección == 'derecha':
        nuevo.place(x=(x_act - ancho_act), y=y_act)
        pos_inic = [x_act - ancho_act]
        distancia = ancho_act
    elif dirección == 'arriba':
        nuevo.place(x=x_act, y=(y_act + altura_act))
        pos_inic = [y_act - altura_act]
        distancia = altura_act
    elif dirección == 'abajo':
        nuevo.place(x=x_act, y=(y_act - altura_act))
        pos_inic = [y_act + altura_act]
        distancia = altura_act
    else:
        raise ValueError
    nuevo.lift()

    deslizar([nuevo], pos_inic, dirección, distancia, paso=0.025, tiempo=0.5)


def quitar(actual, dirección):
    dirección = gf(dirección)
    x_act = actual.winfo_x()
    y_act = actual.winfo_y()
    ancho_act = actual.winfo_width()
    altura_act = actual.winfo_height()

    if dirección == 'izquierda' or dirección == 'derecha':
        pos_inic = [x_act]
        distancia = ancho_act
    elif dirección == 'arriba' or dirección == 'abajo':
        pos_inic = [y_act]
        distancia = altura_act
    else:
        raise ValueError

    deslizar([actual], pos_inic, dirección, distancia, paso=0.025, tiempo=.5)
    actual.lower()


def deslizar(objetos, pos_inic, dirección, distancia, paso=0.025, método='logístico', tiempo=0.5):

    for i in range(1, int(tiempo/paso) + 1):
        t = i*paso
        if método == 'logístico':
            nueva_pos = mat.ceil(distancia/(1+mat.exp(-12/tiempo * (t-tiempo/2))))
        elif método == 'linear':
            nueva_pos = mat.ceil(distancia*t/tiempo)
        else:
            raise ValueError
        if i == int(tiempo/paso):
            nueva_pos = distancia

        if dirección == 'izquierda':
            for n, o in enumerate(objetos):
                o.place(x=pos_inic[n] - nueva_pos)
                o.update()
        elif dirección == 'derecha':
            for n, o in enumerate(objetos):
                o.place(x=pos_inic[n] + nueva_pos)
                o.update()
        elif dirección == 'arriba':
            for n, o in enumerate(objetos):
                o.place(y=pos_inic[n] - nueva_pos)
                o.update()
        elif dirección == 'abajo':
            for n, o in enumerate(objetos):
                o.place(y=pos_inic[n] + nueva_pos)
                o.update()
        else:
            raise ValueError

        time.sleep(paso)



import io
import json
import os
import tkinter as tk
from tkinter import filedialog as diálogo

from tinamit.Conectado import Conectado
from tinamit.definiciones import dir_raíz
from . import CajasGenéricas as CjG
from . import CajasSubEtapas as CjSE
from . import Controles as Ctrl
from . import ControlesGenéricos as CtrG
from . import Formatos as Fm, Botones as Bt, Arte as Art, Animaciones as Anim
from .Formatos import gen_formato as gf


class CajaInic(tk.Frame):
    def __init__(símismo, apli):
        super().__init__(**Fm.formato_CjInic)
        trads = apli.Trads
        símismo.logo = Art.imagen('LogoInic')
        logo = tk.Label(símismo, image=símismo.logo, **Fm.formato_LogoInic)
        logo.pack(**gf(Fm.ubic_LogoInic))

        cj_bts_inic = tk.Frame(símismo, **Fm.formato_cajas)
        Bt.BotónTexto(cj_bts_inic, comanda=símismo.acción_bt_empezar, texto=trads['Empezar'],
                      formato_norm=Fm.formato_BtsInic,
                      formato_sel=Fm.formato_bts_sel,
                      ubicación=gf(Fm.ubic_BtsInic), tipo_ubic='pack')
        Bt.BotónTexto(cj_bts_inic, comanda=símismo.acción_bt_ayuda, texto=trads['Ayuda'],
                      formato_norm=Fm.formato_BtsInic,
                      formato_sel=Fm.formato_bts_sel,
                      ubicación=gf(Fm.ubic_BtsInic), tipo_ubic='pack')
        cj_bts_inic.pack()

        símismo.place(**gf(Fm.ubic_CjInic))

    def acción_bt_empezar(símismo):
        Anim.quitar(símismo, 'arriba')
        símismo.destroy()

    @staticmethod
    def acción_bt_ayuda():
        os.startfile(os.path.join(dir_raíz, 'Documentación.pdf'))


class CajaLeng(tk.Frame):
    def __init__(símismo, apli):
        super().__init__(**Fm.formato_cajas)
        símismo.apli = apli
        símismo.DicLeng = símismo.apli.DicLeng

        símismo.bt_regreso = Bt.BotónImagen(símismo, comanda=símismo.acción_bt_regreso,
                                            img_norm=Art.imagen('BtRegrCent_norm'),
                                            img_sel=Art.imagen('BtRegrCent_sel'),
                                            formato=Fm.formato_botones,
                                            ubicación=gf(Fm.ubic_BtRegrCent), tipo_ubic='place')
        etiq = tk.Label(símismo, text=apli.Trads['OpsLengs'], **Fm.formato_CbzLeng)
        etiq.place(**gf(Fm.ubic_CbzLeng))

        cj_central = tk.Frame(símismo, **Fm.formato_cajas)

        cj_izq = tk.Frame(cj_central, **Fm.formato_cajas)
        etiq_izq = tk.Label(cj_izq, text=apli.Trads['EnTrabajo'], **Fm.formato_EtiqLengLados)
        etiq_izq.place(**gf(Fm.ubic_EtiqCbzColsLeng))
        símismo.lista_izq = CtrG.ListaItemas(cj_izq, formato_cj=Fm.formato_CjLstLengLados,
                                             ubicación=gf(Fm.ubic_LstsLeng), tipo_ubic='place')

        lín_vert_1 = tk.Frame(cj_central, **Fm.formato_LínVert)

        cj_med = tk.Frame(cj_central, **Fm.formato_cajas)
        etiq_med = tk.Label(cj_med, text=apli.Trads['Listas'], **Fm.formato_EtiqLengCentro)
        etiq_med.place(**gf(Fm.ubic_EtiqCbzColsLeng))
        símismo.lista_med = CtrG.ListaItemas(cj_med, formato_cj=Fm.formato_CjLstLengCentro,
                                             ubicación=gf(Fm.ubic_LstsLeng), tipo_ubic='place')

        lín_vert_2 = tk.Frame(cj_central, **Fm.formato_LínVert)

        cj_derech = tk.Frame(cj_central, **Fm.formato_cajas)
        etiq_derech = tk.Label(cj_derech, text=apli.Trads['ParaHacer'], **Fm.formato_EtiqLengLados)
        etiq_derech.place(**gf(Fm.ubic_EtiqCbzColsLeng))
        cj_añadir = Ctrl.CajaAñadirLeng(símismo, cj_derech)
        símismo.lista_derech = CtrG.ListaItemas(cj_derech, formato_cj=Fm.formato_CjLstLengLados,
                                                ubicación=gf(Fm.ubic_LstsLeng_bajo), tipo_ubic='place')

        símismo.establecer_cols()

        cj_izq.place(**gf(Fm.ubic_CjIzqLeng))
        lín_vert_1.place(**gf(Fm.ubic_LínVert1))

        cj_med.place(**gf(Fm.ubic_CjMedLeng))
        lín_vert_2.place(**gf(Fm.ubic_LínVert2))

        cj_añadir.place(**gf(Fm.ubic_CjAñadirLeng))
        cj_derech.place(**gf(Fm.ubic_CjDerchLeng))

        cj_central.place(**gf(Fm.ubic_CjCentLeng))
        símismo.place(**gf(Fm.ubic_CjLeng))

    def acción_bt_regreso(símismo):
        Anim.quitar(símismo, 'derecha')

    def establecer_cols(símismo):
        símismo.DicLeng.verificar_estados()
        for nombre, leng in sorted(símismo.DicLeng.lenguas.items()):
            if 0 < leng['Estado'] < 1:
                lista = símismo.lista_izq
                utilzb = True
            elif leng['Estado'] == 1:
                lista = símismo.lista_med
                utilzb = True
            elif leng['Estado'] == 0:
                lista = símismo.lista_derech
                utilzb = False
            else:
                raise ValueError

            borr = True
            if nombre == símismo.DicLeng.estándar:
                borr = False

            utilznd = False
            if nombre == símismo.DicLeng.dic['Actual']:
                utilznd = True

            Ctrl.ItemaLeng(símismo, lista=lista, nombre=nombre, lengua=leng, estado=leng['Estado'],
                           utilizando=utilznd, utilizable=utilzb, borrable=borr)

    def refrescar(símismo):
        for lista in [símismo.lista_izq, símismo.lista_med, símismo.lista_derech]:
            lista.borrar()
        símismo.establecer_cols()

    def añadir_lengua(símismo, nombre, izqder):
        símismo.DicLeng.lenguas[nombre] = {'Estado': 0.0, 'IzqaDerech': izqder, 'Trads': {}}
        símismo.DicLeng.guardar()
        símismo.refrescar()

    def utilizar(símismo, nombre):
        if nombre != símismo.DicLeng.leng_act:
            if 'AvisoReinic' in símismo.DicLeng.lenguas[nombre]['Trads'].keys():
                texto = símismo.DicLeng.lenguas[nombre]['Trads']['AvisoReinic']
            else:
                texto = símismo.DicLeng.lenguas[símismo.DicLeng.estándar]['Trads']['AvisoReinic']
            Ctrl.CajaAviso(texto=texto, apli=símismo.apli)
            símismo.DicLeng.dic['Actual'] = nombre
            símismo.DicLeng.guardar()
            símismo.refrescar()

    def editar(símismo, nombre):
        leng_base = símismo.DicLeng.dic['Actual']
        if leng_base == nombre:
            leng_base = símismo.DicLeng.estándar
        dic_leng_base = símismo.DicLeng.lenguas[leng_base]

        dic_leng_edit = símismo.DicLeng.lenguas[nombre]
        Ctrl.CajaEditLeng(símismo, símismo.apli, leng_base=leng_base, dic_leng_base=dic_leng_base,
                          leng_edit=nombre, dic_leng_edit=dic_leng_edit)

    def confirmar_borrar(símismo, nombre):
        Ctrl.CajaAvisoBorrar(apli=símismo.apli, nombre=nombre, acción=símismo.borrar)

        símismo.DicLeng.guardar()

    def borrar(símismo, nombre):
        símismo.DicLeng.lenguas.pop(nombre)
        if símismo.DicLeng.dic['Actual'] == nombre:
            símismo.DicLeng.dic['Actual'] = símismo.DicLeng.estándar
        símismo.DicLeng.guardar()
        símismo.refrescar()


class CajaCentral(tk.Frame):
    def __init__(símismo, apli):
        super().__init__(**Fm.formato_CjCent)
        símismo.CjCabeza = CajaCabeza(símismo, apli=apli)

        símismo.ContCjEtapas = CjG.ContCajaEtps(símismo)
        núm_etapas = 4
        símismo.CajasEtapas = [CajaEtp1(símismo.ContCjEtapas, apli, núm_etapas),
                               CajaEtp2(símismo.ContCjEtapas, apli, núm_etapas),
                               CajaEtp3(símismo.ContCjEtapas, apli, núm_etapas),
                               CajaEtp4(símismo.ContCjEtapas, apli, núm_etapas),
                               ]
        símismo.ContCjEtapas.establecer_cajas(símismo.CajasEtapas)

        símismo.CjIzq = CajaIzq(símismo, cajas_etapas=símismo.CajasEtapas)

        símismo.bloquear_cajas(list(range(2, len(símismo.CajasEtapas) + 1)))

        símismo.place(**gf(Fm.ubic_CjCent))

    def bloquear_cajas(símismo, núms_cajas):
        for n in núms_cajas:
            if n > 1:
                símismo.CajasEtapas[n - 2].bloquear_transición(dirección='siguiente')
            if n < len(símismo.CajasEtapas):
                símismo.CajasEtapas[n].bloquear_transición(dirección='anterior')
            símismo.CjIzq.bts[n - 1].bloquear()

    def desbloquear_cajas(símismo, núms_cajas):
        for n in núms_cajas:
            símismo.CajasEtapas[n-1].acción_desbloquear()
            if n > 1:
                símismo.CajasEtapas[n - 2].desbloquear_transición(dirección='siguiente')
            if n < len(símismo.CajasEtapas):
                símismo.CajasEtapas[n].desbloquear_transición(dirección='anterior')
            símismo.CjIzq.bts[n - 1].desbloquear()


class CajaCabeza(tk.Frame):
    def __init__(símismo, pariente, apli):
        super().__init__(pariente, **Fm.formato_CjCabeza)
        símismo.apli = apli
        símismo.pariente = pariente
        símismo.logo_cabeza = Art.imagen('LogoCent')
        logo_cabeza = tk.Label(símismo, image=símismo.logo_cabeza, **Fm.formato_LogoCabz)
        logo_cabeza.place(**gf(Fm.ubic_LogoCabz))

        cj_bts_archivo = tk.Frame(símismo, **Fm.formato_cajas)
        Bt.BotónImagen(cj_bts_archivo, comanda=símismo.acción_bt_nuevo,
                       img_norm=Art.imagen('BtNuevo_norm'),
                       img_sel=Art.imagen('BtNuevo_sel'),
                       formato=Fm.formato_botones,
                       ubicación=gf(Fm.ubic_BtNuevo), tipo_ubic='grid')
        Bt.BotónImagen(cj_bts_archivo, comanda=símismo.acción_bt_guardar,
                       img_norm=Art.imagen('BtGuardar_norm'),
                       img_sel=Art.imagen('BtGuardar_sel'),
                       formato=Fm.formato_botones,
                       ubicación=gf(Fm.ubic_BtGuardar), tipo_ubic='grid')
        Bt.BotónImagen(cj_bts_archivo, comanda=símismo.acción_bt_guardar_como,
                       img_norm=Art.imagen('BtGuardarComo_norm'),
                       img_sel=Art.imagen('BtGuardarComo_sel'),
                       formato=Fm.formato_botones,
                       ubicación=gf(Fm.ubic_BtGuardarComo), tipo_ubic='grid')
        Bt.BotónImagen(cj_bts_archivo, comanda=símismo.acción_bt_abrir,
                       img_norm=Art.imagen('BtAbrir_norm'),
                       img_sel=Art.imagen('BtAbrir_sel'),
                       formato=Fm.formato_botones,
                       ubicación=gf(Fm.ubic_BtAbrir), tipo_ubic='grid')
        cj_bts_archivo.place(**gf(Fm.ubic_BtsArchivo))

        Bt.BotónImagen(símismo, comanda=símismo.acción_bt_leng,
                       img_norm=Art.imagen('BtLeng_norm'),
                       img_sel=Art.imagen('BtLeng_sel'),
                       formato=Fm.formato_botones,
                       ubicación=gf(Fm.ubic_BtLeng), tipo_ubic='place')
        símismo.pack(**gf(Fm.ubic_CjCabeza))

    def acción_bt_nuevo(símismo):
        símismo.apli.Modelo = Conectado()

        símismo.apli.receta = {'conexiones': símismo.apli.Modelo.conexiones,
                               'conv_tiempo': símismo.apli.Modelo.conv_tiempo}

        símismo.pariente.ContCjEtapas.ir_a_caja(1)
        símismo.pariente.desbloquear_cajas([1])
        símismo.pariente.bloquear_cajas([2, 3, 4])

    def acción_bt_guardar(símismo):
        if símismo.apli.ubic_archivo is not None:
            símismo.guardar()
        else:
            símismo.acción_bt_guardar_como()

    def acción_bt_abrir(símismo):
        apli = símismo.apli
        archivo = diálogo.askopenfilename(filetypes=[(apli.Trads['ArchivoTinamit'], '*.tin')],
                                          title=apli.Trads['CargarModeloConectado'])

        if archivo:

            símismo.apli.ubic_archivo = archivo
            with open(archivo, 'r', encoding='utf8') as d:
                receta = json.load(d)

            if 'conv_tiempo' not in receta.keys() or 'conexiones' not in receta.keys():
                Ctrl.CajaAviso(apli=símismo.apli, texto=apli.Trads['ArchivoCorrupto'])

            símismo.apli.Modelo = modelo = Conectado()
            símismo.apli.receta = receta

            try:
                modelo.estab_mds(receta['mds'])
            except KeyError:
                pass

            try:
                modelo.estab_bf(receta['bf'])
            except KeyError:
                pass

            l_mod = list(modelo.modelos)

            for conex in receta['conexiones']:
                mod_fuente = conex['modelo_fuente']

                mod_recip = l_mod[(l_mod.index(mod_fuente) + 1) % 2]

                modelo.conectar_vars(dic_vars={mod_fuente: conex['vars'][mod_fuente],
                                               mod_recip: conex['vars'][mod_recip]},
                                     modelo_fuente=mod_fuente, conv=conex['conv'])

            if len(receta['conv_tiempo']):
                if receta['conv_tiempo'][l_mod[0]] == 1:
                    modelo.estab_conv_tiempo(mod_base=l_mod[0],
                                             conv=receta['conv_tiempo'][l_mod[1]])
                else:
                    modelo.estab_conv_tiempo(mod_base=l_mod[1],
                                             conv=receta['conv_tiempo'][l_mod[0]])

            receta['conexiones'] = modelo.conexiones
            receta['conv_tiempo'] = modelo.conv_tiempo

            símismo.pariente.ContCjEtapas.ir_a_caja(1)
            símismo.pariente.bloquear_cajas([2, 3, 4])
            símismo.pariente.desbloquear_cajas([1])

    def acción_bt_guardar_como(símismo):
        apli = símismo.apli
        archivo = diálogo.asksaveasfilename(defaultextension='.tin',
                                            filetypes=[(apli.Trads['ArchivoTinamit'], '*.tin')],
                                            title=apli.Trads['CargarModeloConectado'])
        símismo.apli.ubic_archivo = archivo
        símismo.guardar()

    def guardar(símismo):
        with io.open(símismo.apli.ubic_archivo, 'w', encoding='utf8') as d:
            json.dump(símismo.apli.receta, d, ensure_ascii=False, sort_keys=True, indent=2)

    def acción_bt_leng(símismo):
        Anim.sobreponer(símismo.apli.CajaCentral, símismo.apli.CajaLenguas, 'izquierda')


class CajaIzq(tk.Frame):
    def __init__(símismo, pariente, cajas_etapas):
        super().__init__(pariente, **Fm.formato_cajas)

        símismo.bts = []
        for cj in cajas_etapas:
            símismo.bts.append(Bt.BotónNavIzq(símismo, caja=cj))

        símismo.pack(**gf(Fm.ubic_CjIzq))


class CajaEtp1(CjG.CajaEtapa):
    def __init__(símismo, pariente, apli, total):
        super().__init__(pariente, nombre=apli.Trads['CargarModelos'], núm=1, total=total)

        total_subcajas = 1
        subcajas = [CjSE.CajaSubEtp11(símismo, apli, total=total_subcajas)]

        símismo.especificar_subcajas(subcajas)

    def acción_desbloquear(símismo):
        símismo.desbloquear_subcajas([1])


class CajaEtp2(CjG.CajaEtapa):
    def __init__(símismo, pariente, apli, total):
        super().__init__(pariente, nombre=apli.Trads['ConectarVariables'], núm=2, total=total)
        total_subcajas = 1
        subcajas = [CjSE.CajaSubEtp21(símismo, apli, total=total_subcajas)]

        símismo.especificar_subcajas(subcajas)

    def acción_desbloquear(símismo):
        símismo.desbloquear_subcajas([1])


class CajaEtp3(CjG.CajaEtapa):
    def __init__(símismo, pariente, apli, total):
        super().__init__(pariente, nombre=apli.Trads['Simular'], núm=3, total=total)

        total_subcajas = 1
        subcajas = [CjSE.CajaSubEtp31(símismo, apli, total=total_subcajas)]

        símismo.especificar_subcajas(subcajas)

    def acción_desbloquear(símismo):
        símismo.desbloquear_subcajas([1])


class CajaEtp4(CjG.CajaEtapa):
    def __init__(símismo, pariente, apli, total):
        super().__init__(pariente, nombre=apli.Trads['AnálisisIncert'], núm=4, total=total)
        total_subcajas = 1
        subcajas = [CjSE.CajaSubEtp41(símismo, apli, total=total_subcajas)]

        símismo.especificar_subcajas(subcajas)

    def acción_desbloquear(símismo):
        símismo.desbloquear_subcajas([1])



import tkinter as tk

from . import Arte as Gr
from . import Formatos as Fm
from .Formatos import gen_formato as gf


class Botón(object):
    def __init__(símismo, pariente, comanda=None, formato=None, img_norm=None, img_sel=None, img_bloq=None,
                 texto=None, formato_norm=None, formato_sel=None, formato_bloq=None,
                 ubicación=None, tipo_ubic=None, tipo_combin='left'):

        símismo.formato = {}
        if tipo_combin is not None:
            símismo.formato['compound'] = tipo_combin
        if texto is not None:
            símismo.formato['text'] = texto
        if formato is not None:
            for ll, v in formato.items():
                símismo.formato[ll] = v

        símismo.formato_norm = {}
        símismo.formato_bloq = {}
        símismo.formato_sel = {}

        if img_norm is not None:
            símismo.formato_norm['image'] = img_norm
        if img_sel is not None:
            símismo.formato_sel['image'] = img_sel
        if img_bloq is not None:
            símismo.formato_bloq['image'] = img_bloq
        if formato_norm is not None:
            for ll, v in formato_norm.items():
                símismo.formato_norm[ll] = v
        if formato_sel is not None:
            for ll, v in formato_sel.items():
                símismo.formato_sel[ll] = v
        if formato_bloq is not None:
            for ll, v in formato_bloq.items():
                símismo.formato_bloq[ll] = v

        símismo.estado = 'Normal'

        # Hasta que aprende cómo cargar Python 3.5 en esta compu...
        dic_formato = símismo.formato.copy()
        dic_formato.update(símismo.formato_norm)

        símismo.bt = tk.Button(pariente, relief=tk.FLAT, command=comanda,
                               **dic_formato)

        símismo.bloquear()
        símismo.desbloquear()
        símismo.bt.bind('<Enter>', lambda event, b=símismo: b.resaltar())
        símismo.bt.bind('<Leave>', lambda event, b=símismo: b.desresaltar())

        if tipo_ubic == 'pack':
            símismo.bt.pack(**ubicación)
        elif tipo_ubic == 'place':
            símismo.bt.place(**ubicación)
        elif tipo_ubic == 'grid':
            símismo.bt.grid(**ubicación)

    def bloquear(símismo):
        símismo.estado = 'Bloqueado'
        símismo.bt.configure(cursor='X_cursor', state='disabled', **símismo.formato_bloq)

    def desbloquear(símismo):
        símismo.estado = 'Normal'
        símismo.bt.configure(cursor='arrow', state='normal', **símismo.formato_norm)

    def resaltar(símismo):
        if símismo.estado == 'Normal':
            símismo.bt.configure(**símismo.formato_sel)

    def desresaltar(símismo):
        if símismo.estado == 'Normal':
            símismo.bt.configure(**símismo.formato_norm)

    def seleccionar(símismo):
        if símismo.estado == 'Normal':
            símismo.bt.configure(**símismo.formato_sel)

    def deseleccionar(símismo):
        if símismo.estado == 'Normal':
            símismo.bt.configure(**símismo.formato_norm)

    def estab_comanda(símismo, comanda):
        símismo.bt.config(command=comanda)


class BotónTexto(Botón):
    def __init__(símismo, pariente, texto, formato_norm, formato_sel,
                 ubicación, tipo_ubic, comanda=None, formato_bloq=None):

        super().__init__(pariente, comanda, texto=texto, formato_norm=formato_norm,
                         formato_sel=formato_sel, formato_bloq=formato_bloq,
                         ubicación=ubicación, tipo_ubic=tipo_ubic)


class BotónImagen(Botón):
    def __init__(símismo, pariente, comanda, formato, img_norm, img_sel, img_bloq=None,
                 ubicación=None, tipo_ubic=None):
        super().__init__(pariente, comanda=comanda, formato=formato, img_norm=img_norm,
                         img_sel=img_sel, img_bloq=img_bloq,
                         ubicación=ubicación, tipo_ubic=tipo_ubic)


class BotónNavIzq(object):
    def __init__(símismo, pariente, caja):
        img_norm = Gr.imagen('BtNavIzq_%i_norm' % caja.núm)
        img_bloq = Gr.imagen('BtNavIzq_%i_bloq' % caja.núm)
        img_sel = Gr.imagen('BtNavIzq_%i_sel' % caja.núm)

        cj = tk.Frame(pariente, **Fm.formato_cajas)
        símismo.bt = BotónImagen(pariente=cj, comanda=caja.traer_me, img_norm=img_norm, img_bloq=img_bloq,
                                 img_sel=img_sel, ubicación=gf(Fm.ubic_BtNavIzq), tipo_ubic='pack',
                                 formato=Fm.formato_BtsNavIzq)

        símismo.color = Fm.color_bts[str(caja.núm)]
        símismo.lín = tk.Frame(cj, bg=símismo.color, **Fm.formato_lín_bts)

        símismo.lín.pack(**gf(Fm.ubic_LínNavIzq))
        cj.pack(**gf(Fm.ubic_CjBtNavIzq))

    def desbloquear(símismo):
        símismo.bt.desbloquear()
        símismo.lín.configure(bg=símismo.color)

    def bloquear(símismo):
        símismo.bt.bloquear()
        símismo.lín.configure(bg=Fm.color_gris)


class BotónNavEtapa(BotónImagen):
    def __init__(símismo, pariente, tipo):
        if tipo == 'adelante':
            img_norm = Gr.imagen('BtNavEtp_adel_norm')
            img_bloq = Gr.imagen('BtNavEtp_adel_bloq')
            img_sel = Gr.imagen('BtNavEtp_adel_sel')
            ubicación = gf(Fm.ubic_BtNavEtp_adel)
            comanda = pariente.ir_etp_siguiente
        elif tipo == 'atrás':
            img_norm = Gr.imagen('BtNavEtp_atrs_norm')
            img_bloq = Gr.imagen('BtNavEtp_atrs_bloq')
            img_sel = Gr.imagen('BtNavEtp_atrs_sel')
            ubicación = gf(Fm.ubic_BtNavEtp_atrs)
            comanda = pariente.ir_etp_anterior
        else:
            raise ValueError

        super().__init__(pariente=pariente, comanda=comanda, img_norm=img_norm, img_bloq=img_bloq,
                         img_sel=img_sel, ubicación=ubicación, tipo_ubic='place',
                         formato=Fm.formato_BtsNavEtapa)


class BotónNavSub(BotónImagen):
    def __init__(símismo, pariente, tipo):
        if tipo == 'adelante':
            img_norm = Gr.imagen('BtNavSub_adel_norm')
            img_bloq = Gr.imagen('BtNavSub_adel_bloq')
            img_sel = Gr.imagen('BtNavSub_adel_sel')
            ubicación = gf(Fm.ubic_BtNavSub_adel)
            comanda = pariente.ir_sub_siguiente
        elif tipo == 'atrás':
            img_norm = Gr.imagen('BtNavSub_atrs_norm')
            img_bloq = Gr.imagen('BtNavSub_atrs_bloq')
            img_sel = Gr.imagen('BtNavSub_atrs_sel')
            ubicación = gf(Fm.ubic_BtNavSub_atrs)
            comanda = pariente.ir_sub_anterior
        else:
            raise ValueError

        super().__init__(pariente=pariente, comanda=comanda, img_norm=img_norm, img_bloq=img_bloq,
                         img_sel=img_sel, ubicación=ubicación, tipo_ubic='place',
                         formato=Fm.formato_BtsNavSub)


class BotónAltern(BotónImagen):
    def __init__(símismo, pariente, formato, img_1, img_2, img_bloq=None, comanda_segundaria=None,
                 ubicación=None, tipo_ubic=None):
        super().__init__(pariente, comanda=símismo.acción, formato=formato, img_norm=img_1,
                         img_sel=img_2, img_bloq=img_bloq,
                         ubicación=ubicación, tipo_ubic=tipo_ubic)
        símismo.img_1 = img_1
        símismo.img_2 = img_2
        símismo.comanda_segundaria = comanda_segundaria

        símismo.val = True

    def acción(símismo):
        símismo.poner(not símismo.val)
        if símismo.comanda_segundaria is not None:
            símismo.comanda_segundaria()

    def borrar(símismo):
        símismo.poner(True)

    def poner(símismo, valor):
        símismo.val = valor

        if símismo.val:
            símismo.formato_sel['image'] = símismo.img_2
            símismo.formato_norm['image'] = símismo.img_1
        else:
            símismo.formato_sel['image'] = símismo.img_1
            símismo.formato_norm['image'] = símismo.img_2

        símismo.bt.configure(image=símismo.formato_norm['image'])



import tkinter as tk

from tinamit.Interfaz import Cajas as Cj
from tinamit.Interfaz import Formatos as Fm
from tinamit.Interfaz import Traducciones as Trad
from tinamit.Conectado import Conectado


class Apli(tk.Frame):
    """
    Aquí creamos la aplicación centra para el IGU.
    """
    def __init__(símismo, pariente):
        tk.Frame.__init__(símismo, pariente)
        pariente.title('Tinamit')

        pariente.geometry('%ix%i' % (Fm.ancho_ventana, Fm.altura_ventana))
        pariente.configure(background=Fm.col_fondo)
        pariente.minsize(width=Fm.ancho_ventana, height=Fm.altura_ventana)

        símismo.Modelo = Conectado()

        símismo.DicLeng = Trad.Diccionario()
        símismo.Trads = símismo.DicLeng.trads_act
        símismo.receta = {'conexiones': símismo.Modelo.conexiones,
                          'conv_tiempo': símismo.Modelo.conv_tiempo}
        símismo.ubic_archivo = None

        símismo.CajaInic = Cj.CajaInic(símismo)
        símismo.CajaCentral = Cj.CajaCentral(símismo)
        símismo.CajaLenguas = Cj.CajaLeng(símismo)
        símismo.CajaCentral.lift()
        símismo.CajaInic.lift()
        símismo.pack()


def correr():
    """
    Esta función empieza el interfaz.
    """

    raíz = tk.Tk()
    apli = Apli(raíz)
    apli.mainloop()


# Si este archivo se ejecuta directamente (y no se importa), empezar el IGU.
if __name__ == '__main__':
    correr()



import tkinter as tk

from . import Animaciones as Anim
from . import Botones as Bt
from . import ControlesGenéricos as Ctrl
from . import Formatos as Fm
from .Formatos import gen_formato as gf


class ContCajaEtps(tk.Frame):
    def __init__(símismo, pariente):
        super().__init__(pariente, **Fm.formato_cajas)
        símismo.pariente = pariente

        símismo.Cajas = []
        símismo.CajaActual = None
        símismo.pack(**gf(Fm.ubic_CjContCjEtps))
        símismo.en_transición = False

    def establecer_cajas(símismo, cajas_etapas):
        for n, cj in enumerate(cajas_etapas):
            símismo.Cajas.append(cj)
        símismo.CajaActual = símismo.Cajas[0]
        símismo.CajaActual.lift()

    def ir_a_caja(símismo, núm_cj_nueva):
        if núm_cj_nueva < símismo.CajaActual.núm:
            dirección = 'abajo'
        elif núm_cj_nueva > símismo.CajaActual.núm:
            dirección = 'arriba'
        else:
            return

        if not símismo.en_transición:
            símismo.en_transición = True
            nueva_caja = símismo.Cajas[núm_cj_nueva-1]
            nueva_caja.lift()
            Anim.intercambiar(símismo.CajaActual, nueva_caja, dirección=dirección)
            símismo.CajaActual = nueva_caja
            símismo.en_transición = False

    def bloquear_cajas(símismo, núms_cajas):
        símismo.pariente.bloquear_cajas(núms_cajas)

    def desbloquear_cajas(símismo, núms_cajas):
        símismo.pariente.desbloquear_cajas(núms_cajas)


class CajaEtapa(tk.Frame):
    def __init__(símismo, pariente, nombre, núm, total):
        super().__init__(pariente, **Fm.formato_cajas)
        símismo.núm = núm
        símismo.pariente = pariente
        símismo.SubCajas = []
        símismo.SubCajaActual = None
        
        etiq = tk.Label(símismo, text=nombre, **Fm.formato_EncbzCjEtp)
        etiq.place(**gf(Fm.ubic_EncbzCjEtp))

        símismo.BtAtrás = símismo.BtAdelante = None
        if núm > 1:
            símismo.BtAtrás = Bt.BotónNavEtapa(símismo, tipo='atrás')
        if núm < total:
            símismo.BtAdelante = Bt.BotónNavEtapa(símismo, tipo='adelante')

        símismo.place(**gf(Fm.ubic_CjEtp))

    def especificar_subcajas(símismo, subcajas):
        símismo.SubCajas = subcajas
        símismo.ir_a_sub(1)

    def bloquear_cajas(símismo, núms_cajas):
        símismo.pariente.bloquear_cajas(núms_cajas)

    def desbloquear_cajas(símismo, núms_cajas):
        símismo.pariente.desbloquear_cajas(núms_cajas)

    def bloquear_transición(símismo, dirección):
        if dirección == 'anterior' and símismo.BtAtrás is not None:
            símismo.BtAtrás.bloquear()
        elif dirección == 'siguiente' and símismo.BtAdelante is not None:
            símismo.BtAdelante.bloquear()

    def desbloquear_transición(símismo, dirección):
        if dirección == 'anterior' and símismo.BtAtrás is not None:
            símismo.BtAtrás.desbloquear()
        elif dirección == 'siguiente' and símismo.BtAdelante is not None:
            símismo.BtAdelante.desbloquear()

    def ir_a_sub(símismo, núm_sub_nueva):
        if símismo.SubCajaActual is None:
            símismo.SubCajaActual = símismo.SubCajas[núm_sub_nueva-1]
            símismo.SubCajaActual.lift()
        else:
            if núm_sub_nueva < símismo.SubCajaActual.núm:
                dirección = 'derecha'
            elif núm_sub_nueva > símismo.SubCajaActual.núm:
                dirección = 'izquierda'
            else:
                return
            Anim.intercambiar(símismo.SubCajaActual, símismo.SubCajas[núm_sub_nueva-1], dirección=dirección)
            símismo.SubCajaActual = símismo.SubCajas[núm_sub_nueva-1]

    def traer_me(símismo):
        símismo.pariente.ir_a_caja(símismo.núm)

    def ir_etp_siguiente(símismo):
        símismo.pariente.ir_a_caja(símismo.núm + 1)

    def ir_etp_anterior(símismo):
        símismo.pariente.ir_a_caja(símismo.núm - 1)
        
    def bloquear_subcajas(símismo, núms_cajas):
        for n in núms_cajas:
            if n > 1:
                símismo.SubCajas[n - 2].bloquear_transición(dirección='siguiente')
            if n < len(símismo.SubCajas):
                símismo.SubCajas[n].bloquear_transición(dirección='anterior')

    def desbloquear_subcajas(símismo, núms_cajas):
        for n in núms_cajas:
            símismo.SubCajas[n - 1].acción_desbloquear()
            if n > 1:
                símismo.SubCajas[n - 2].desbloquear_transición(dirección='siguiente')
            if n < len(símismo.SubCajas):
                símismo.SubCajas[n].desbloquear_transición(dirección='anterior')

    def acción_desbloquear(símismo):
        pass


class CajaSubEtapa(tk.Frame):
    def __init__(símismo, pariente, nombre, núm, total):
        super().__init__(pariente, **Fm.formato_cajas)

        símismo.pariente = pariente
        símismo.núm = núm
        
        if nombre is not None:
            etiq = tk.Label(símismo, text=nombre, **Fm.formato_EncbzCjSubEtp)
            etiq.place(**gf(Fm.ubic_EncbzCjSubEtp))
            
        símismo.BtAdelante = None
        símismo.BtAtrás = None
        if núm < total:
            símismo.BtAdelante = Bt.BotónNavSub(símismo, tipo='adelante')
        if núm > 1:
            símismo.BtAtrás = Bt.BotónNavSub(símismo, tipo='atrás')

        símismo.place(gf(Fm.ubic_CjSubEtp))

    def bloquear_transición(símismo, dirección):
        if dirección == 'anterior' and símismo.BtAtrás is not None:
            símismo.BtAtrás.bloquear()
        elif dirección == 'siguiente' and símismo.BtAdelante is not None:
            símismo.BtAdelante.bloquear()

    def desbloquear_transición(símismo, dirección):
        if dirección == 'anterior' and símismo.BtAtrás is not None:
            símismo.BtAtrás.desbloquear()
        elif dirección == 'siguiente' and símismo.BtAdelante is not None:
            símismo.BtAdelante.desbloquear()

    def ir_sub_siguiente(símismo):
        símismo.pariente.ir_a_sub(símismo.núm + 1)

    def ir_sub_anterior(símismo):
        símismo.pariente.ir_a_sub(símismo.núm - 1)

    def ir_etp_siguiente(símismo):
        símismo.pariente.ir_etp_siguiente()

    def ir_etp_anterior(símismo):
        símismo.pariente.ir_etp_anterior()

    def acción_desbloquear(símismo):
        pass  # A implementar en subcajas específicas donde necesario

    def verificar_completo(símismo):
        pass  # A implementar en subcajas específicas donde necesario


class CajaActivable(tk.Frame):
    def __init__(símismo, pariente, ubicación, tipo_ubic):
        super().__init__(pariente, **Fm.formato_cajas)
        símismo.etiquetas = []
        símismo.colores_etiquetas = []
        símismo.botones = []
        
        if tipo_ubic == 'pack':
            símismo.pack(**ubicación)
        elif tipo_ubic == 'place':
            símismo.place(**ubicación)

    def especificar_objetos(símismo, objetos):
        símismo.etiquetas = [x for x in objetos if type(x) is tk.Label]
        símismo.botones = [x for x in objetos if
                           type(x) is Bt.BotónImagen or
                           type(x) is Bt.BotónTexto or
                           type(x) is Ctrl.Menú]

        símismo.colores_etiquetas = [x.cget['fg'] for x in símismo.etiquetas]

    def bloquear(símismo):
        for etiq in símismo.etiquetas:
            etiq.configure(Fm.formato_EtiqCtrl_bloq)
        for bt in símismo.botones:
            bt.bloquear()

    def desbloquear(símismo):
        for n, etiq in enumerate(símismo.etiquetas):
            etiq.configure(color=símismo.colores_etiquetas[n])
        for bt in símismo.botones:
            bt.acción_desbloquear()



import tkinter as tk

from tinamit.Interfaz import Formatos as Fms


class ConecciónVar(object):
    def __init__(símismo, pariente, var1, var2, multiplicador):
        símismo.var1 = var1
        símismo.var2 = var2
        símismo.multiplicador = multiplicador
        símismo.caja = tk.Frame(pariente, **Fms.formato_cajas)


        símismo.botón_quitar = BotónQuitarConección(símismo)
        símismo.botón_editar = BotónEditarConección(símismo)


class BotónQuitarConección(object):
    pass


class BotónEditarConección(object):
    pass


class Lengua(object):
    def __init__(símismo):
        pass



from pkg_resources import resource_filename

__author__ = 'Julien Malard'

__notes__ = 'Contacto: julien.malard@mail.mcgill.ca'


with open(resource_filename('tinamit', 'versión.txt')) as archivo_versión:
    versión = archivo_versión.read().strip()

__version__ = versión



import json
import pkg_resources
import re

# Buscar el archivo de conversiones.
archivo_json = pkg_resources.resource_filename('tinamit.Unidades', 'equiv_unid.json')

# Leer las conversiones.
with open(archivo_json, 'r', encoding='utf8') as d:
    dic_doc = json.load(d)
    dic_conv = dic_doc['conv']
    dic_equiv = dic_doc['equiv']


def convertir(de, a,  val=1, clase=None, lengua=None):
    """
    Esta función convierte un valor de una unidad a otra.

    :param de: La unidad original.
    :type de: str

    :param a: La unidad final.
    :type a: str

    :param clase: El tipo de unidad ('tiempo', 'distancia', etc.) No es obligatoria.
    :type clase: str

    :param val: El valor para convertir.
    :type val: float

    :param lengua: La lengua en la cuál se están especificando las unidades.
    :type lengua: str

    :return: El valor convertido.
    :rtype: float

    """

    # Estar seguro que diferencias en mayúsculos o espacios no causen problemas
    de = de.lower().replace(' ', '')
    a = a.lower().replace(' ', '')

    # Si son las mismas unidades, no hay nada que hacer, por supuesto.
    if de == a:
        return val

    # Asegurarse que la clase de unidades sea una lista.
    if clase is None:
        # Si no se especificó nada, buscar en todas las clases.
        clases = list(dic_conv)
    elif type(clase) is list:
        clases = clase
    else:
        clases = [clase]

    if lengua is None:
        lenguas = list(dic_equiv)
    elif type(lengua) is list:
        lenguas = lengua
    else:
        lenguas = [lengua]

    #
    de = 'm2/s'
    re_unid = r'(?P<unid>[^\W\d]+)(?P<exp>\-?(?=\d)?[\d]*)'
    regex = r'{}+(?P<oper>[\*\/]?)'.format(re_unid)
    unidades_simp = re.finditer(regex, de)
    lista = list(unidades_simp)
    for l in lista:
        print(l.group('unid'))
        print(l.group('exp'))
        print(l.group('oper'))

    factor = 1
    for u in unidades_simp:
        conv = convertir_unid_senc(de=u['de'], a=u['a'], clases=clases, lenguas=lenguas)
        factor *= conv**u['exp']

    return factor * val


def convertir_unid_senc(de, a, clases, lenguas):
    """
    Esta función convierte una unidad sencilla a otra.

    :param de: La unidad original.
    :type de: str

    :param a: La unidad final.
    :type a: str

    :param clases: El tipo de unidad ('tiempo', 'distancia', etc.)
    :type clases: list

    :param lenguas: La lengua en la cuál se están especificando las unidades.
    :type lenguas: list

    :return: El valor convertido.
    :rtype: float

    """

    for leng in lenguas:
        for c in clases:
            try:
                for unid, equivs in dic_equiv[leng][c].items():
                    if de in equivs:
                        de = unid
                    if a in equivs:
                        a = unid
            except KeyError:
                pass

    factor = None

    for c in clases:
        try:
            dic = dic_conv[c]
        except KeyError:
            raise KeyError('Clase de unidades "{}" no reconocida. Debe ser uno de:\n{}.'.format(
                clases,
                ', '.join(list(dic_conv))
            ))

        if de in dic:
            if a in dic[de]:
                factor = dic[de][a]
        if factor is None:
            if a in dic:
                if de in dic[a]:
                    factor = 1/dic[a][de]

        if factor is None:
            # Verificar si las dos unidades tendría conexión a una misma tercera unidad (por ejemplo,
            # cm y mm están vinculados a m, y por su relacion a m se podría establecer la relación entre ambos.)
            for unid in dic:
                if a in dic[unid] and de in dic[unid]:
                    factor = dic[unid][a] / dic[unid][de]
                    break

        if factor is not None:
            break

    if factor is None:
        raise ValueError('Las unidades de tiempo de los dos modelos ({} y {}) son incompatibles. Si'
                         'te parece irrazonable, puedes quejarte a nosotros.'.format(de, a))

    return factor



import ctypes
import os
import struct

from tinamit.Modelo import Modelo


class EnvolturaMDS(Modelo):
    """
    Esta clase sirve para representar modelo de dinámicas de los sistemas (MDS). Se debe crear una subclase para cada
    tipo de MDS. Al moment, el único incluido es VENSIM.
    """

    def __init__(símismo):
        # Modelos DS se identifican por el nombre 'mds'.
        super().__init__(nombre='mds')

    def inic_vars(símismo):
        """
        Este método se deja a las subclases de :class:`~tinamit.MDS.EnvolturaMDS` para implementar.

        Ver :func:`Modelo.Modelo.inic_vars` para más información.
        """

        raise NotImplementedError

    def obt_unidad_tiempo(símismo):
        """
        Cada envoltura de programa DS debe implementar eset metodo para devolver las unidades de tiempo del modelo DS
        cargado.

        :return: Las unidades del modelo DS cargado.
        :rtype: str

        """
        raise NotImplementedError

    def cambiar_vals_modelo_interno(símismo, valores):
        """
        Este método se deja a las subclases de :class:`~tinamit.MDS.EnvolturaMDS` para implementar.

        Ver :func:`Modelo.Modelo.cambiar_vals_modelo` para más información.

        :param valores: El diccionario de valores para cambiar.
        :type valores: dict

        """
        raise NotImplementedError

    def incrementar(símismo, paso):
        """
        Este método se deja a las subclases de :class:`~tinamit.MDS.EnvolturaMDS` para implementar.

        Debe avanzar la simulación del modelo DS de ``paso`` unidades de tiempo.  Ver
        :func:`Modelo.Modelo.incrementar` para más información.

        :param paso: El paso con cual incrementar el modelo.
        :type paso: int

        """
        raise NotImplementedError

    def leer_vals(símismo):
        """
        Este método se deja a las subclases de :class:`~tinamit.MDS.EnvolturaMDS` para implementar.

        Debe leer los valores de los variables en el modelo MDS. Si es más fácil, puede simplemente leer los valores
        de los variables que están en la lista ``EnvolturaMDS.vars_saliendo`` (los variables del DS que están
        conectados con el modelo biofísico).

        Ver :func:`Modelo.Modelo.leer_vals` para más información.

        """
        raise NotImplementedError

    def iniciar_modelo(símismo, nombre_corrida, tiempo_final):
        """
        Este método se deja a las subclases de :class:`~tinamit.MDS.EnvolturaMDS` para implementar.

        Ver :func:`Modelo.Modelo.iniciar_modelo` para más información.

        :param nombre_corrida: El nombre de la corrida (útil para guardar datos).
        :type nombre_corrida: str

        :param tiempo_final: El tiempo final de la simulación.
        :type tiempo_final: int

        """
        raise NotImplementedError

    def cambiar_var(símismo, var, val):
        """
        Est método cambia el valor inicial de un variable (antes de empezar la simulación). Se emplea principalmente
        para activar y desactivar políticas.

        :param var: El nombre del variable para cambiar.
        :type var: str
        :param val: El nuevo valor del variable.
        :type val: float

        """

        # Simplemente se emplea la función Modelo.cambiar_vals().
        símismo.cambiar_vals(valores={var: val})

    def cerrar_modelo(símismo):
        """
        Este método se deja a las subclases de :class:`~tinamit.MDS.EnvolturaMDS` para implementar.

        Debe llamar acciones necesarias para terminar la simulación y cerrar el modelo DS, si aplican.

        Ver :func:`Modelo.Modelo.cerrar_modelo` para más información.

        """
        raise NotImplementedError


class ModeloVENSIM(EnvolturaMDS):
    """
    Esta es la envoltura para modelos de tipo VENSIM. Puede leer y controlar (casi) cualquier modelo VENSIM para que
    se pueda emplear en Tinamit.
    Necesitarás la versión DSS de VENSIM para que funcione en Tinamit.
    """

    def __init__(símismo, archivo):
        """
        La función de inicialización del modelo. Creamos el vínculo con el DLL de VENSIM y cargamos el modelo
        especificado.

        :param archivo: El archivo del modelo que queieres cargar, en formato .vpm.
        :type archivo: str
        """

        # Llamar el DLL de VENSIM.
        símismo.dll = dll = ctypes.WinDLL('C:\\Windows\\System32\\vendll32.dll')

        # Inicializar Vensim
        símismo.comanda_vensim(func=dll.vensim_command,
                               args=[''],
                               mensaje_error='Error iniciando VENSIM.')

        # Cargar el modelo
        símismo.comanda_vensim(func=dll.vensim_command,
                               args='SPECIAL>LOADMODEL|%s' % archivo,
                               mensaje_error='Eroor cargando el modelo de VENSIM.')

        # Parámetros estéticos de ejecución.
        símismo.comanda_vensim(func=dll.vensim_be_quiet, args=[2],
                               mensaje_error='Error en la comanda "vensim_be_quiet".',
                               val_error=-1)

        # Inicializar ModeloVENSIM como una EnvolturaMDS.
        super().__init__()

    def inic_vars(símismo):
        """
        Inicializamos el diccionario de variables del modelo VENSIM.
        """

        # Primero, verificamos el tamañano de memoria necesario para guardar una lista de los nombres de los variables.

        mem = ctypes.create_string_buffer(0)  # Crear una memoria intermedia

        # Verificar el tamaño necesario
        tamaño_nec = símismo.comanda_vensim(func=símismo.dll.vensim_get_varnames,
                                            args=['*', 0, mem, 0],
                                            mensaje_error='Error obteniendo eñ tamaño de los variables VENSIM.',
                                            val_error=-1, devolver=True
                                            )

        mem = ctypes.create_string_buffer(tamaño_nec)  # Una memoria intermedia con el tamaño apropiado

        # Guardar y decodar los nombres de los variables.
        símismo.comanda_vensim(func=símismo.dll.vensim_get_varnames,
                               args=['*', 0, mem, tamaño_nec],
                               mensaje_error='Error obteniendo los nombres de los variables de VENSIM.',
                               val_error=-1
                               )
        variables = [x for x in mem.raw.decode().split('\x00') if x]

        # Quitar los nombres de variables VENSIM genéricos de la lista.
        for i in ['FINAL TIME', 'TIME STEP', 'INITIAL TIME', 'SAVEPER', 'Time']:
            if i in variables:
                variables.remove(i)

        # Sacar los nombres de variables editables únicamente
        mem = ctypes.create_string_buffer(tamaño_nec)
        símismo.comanda_vensim(func=símismo.dll.vensim_get_varnames,
                               args=['*', 12, mem, tamaño_nec],
                               mensaje_error='Error obteniendo los nombres de los variables editables ("Gaming") de '
                                             'VENSIM.',
                               val_error=-1
                               )

        editables = [x for x in mem.raw.decode().split('\x00') if x]

        # Sacar las unidades de los variables, e identificar los variables constantes
        unidades = {}
        constantes = []
        for var in variables:
            # Para cada variable...

            # Sacar sus unidades
            mem = ctypes.create_string_buffer(50)
            símismo.comanda_vensim(func=símismo.dll.vensim_get_varattrib,
                                   args=[var, 1, mem, 50],
                                   mensaje_error='Error obteniendo las unidades del variable {} en VENSIM'.format(var),
                                   val_error=-1
                                   )
            # Guardamos las unidades en un diccionario temporario.
            unidades[var] = mem.raw.decode().split('\x00')[0]

            # Verificar si el variable es un constante (ingreso)
            mem = ctypes.create_string_buffer(50)
            símismo.comanda_vensim(func=símismo.dll.vensim_get_varattrib,
                                   args=[var, 14, mem, 50],
                                   mensaje_error='Error obteniendo la clase del variable {} en VENSIM'.format(var),
                                   val_error=-1)

            tipo_var = mem.raw.decode()

            # Guardamos los variables constantes en una lista.
            if tipo_var == 'Constant':
                constantes.append(var)

        # Actualizar el diccionario de variables.
        for var in variables:
            # Para cada variable, creamos un diccionario especial, con su valor y unidades. Puede ser un variable
            # de ingreso si es de tipo editable ("Gaming"), y puede ser un variable de egreso si no es un valor
            # constante.
            dic_var = {'val': None,
                       'unidades': unidades[var],
                       'ingreso': var in editables,
                       'egreso': var not in constantes}

            # Guadar el diccionario del variable en el diccionario general de variables.
            símismo.variables[var] = dic_var

    def obt_unidad_tiempo(símismo):
        """
        Aquí, sacamos las unidades de tiempo del modelo VENSIM.

        :return: Las unidades de tiempo.
        :rtype: str

        """

        # Una memoria intermediaria.
        mem_inter = ctypes.create_string_buffer(50)

        # Sacar las unidades del variable "TIME STEP".
        símismo.comanda_vensim(func=símismo.dll.vensim_get_varattrib,
                               args=['TIME STEP', 1, mem_inter, 50],
                               mensaje_error='Error obteniendo las unidades de tiempo de VENSIM.',
                               val_error=-1)

        # Decodar
        unidades = mem_inter.raw.decode().split('\x00')[0]

        return unidades

    def iniciar_modelo(símismo, tiempo_final, nombre_corrida):
        """
        Acciones necesarias para iniciar el modelo VENSIM.

        :param nombre_corrida: El nombre de la corrida del modelo.
        :type nombre_corrida: str

        :param tiempo_final: El tiempo final de la simulación.
        :type tiempo_final: int

        """

        # Establecer el nombre de la corrida.
        símismo.comanda_vensim(func=símismo.dll.vensim_command,
                               args="SIMULATE>RUNNAME|%s" % nombre_corrida,
                               mensaje_error='Error iniciando la corrida VENSIM.')

        # Establecer el tiempo final.
        símismo.comanda_vensim(func=símismo.dll.vensim_command,
                               args='SIMULATE>SETVAL|%s = %i' % ('FINAL TIME', tiempo_final),
                               mensaje_error='Error estableciendo el tiempo final para VENSIM.')

        # Iniciar la simulación en modo juego ("Game"). Esto permitirá la actualización de los valores de los variables
        # a través de la simulación.
        símismo.comanda_vensim(func=símismo.dll.vensim_command,
                               args="MENU>GAME",
                               mensaje_error='Error inicializando el juego VENSIM.')

    def cambiar_vals_modelo_interno(símismo, valores):
        """
        Esta función cambiar los valores de variables en VENSIM. Notar que únicamente los variables identificados como
        de tipo "Gaming" en el modelo podrán actualizarse.

        :param valores: Un diccionario de variables y sus nuevos valores.
        :type valores: dict

        """

        for var, val in valores.items():
            # Para cada variable para cambiar...

            # Actualizar el valor en el modelo VENSIM.
            símismo.comanda_vensim(func=símismo.dll.vensim_command,
                                   args='SIMULATE>SETVAL|%s = %f' % (var, val),
                                   mensaje_error='Error cambiando el variable %s.' % var)

    def incrementar(símismo, paso):
        """
        Esta función avanza la simulación VENSIM de ``paso`` pasos.

        :param paso: El número de pasos para tomar.
        :type paso: int

        """

        # Establecer el paso.
        símismo.comanda_vensim(func=símismo.dll.vensim_command,
                               args="GAME>GAMEINTERVAL|%i" % paso,
                               mensaje_error='Error estableciendo el paso de VENSIM.')

        # Avanzar el modelo.
        símismo.comanda_vensim(func=símismo.dll.vensim_command,
                               args="GAME>GAMEON", mensaje_error='Error para incrementar VENSIM.')

    def leer_vals(símismo):
        """
        Este método lee los valores intermediaros de los variables del modelo VENSIM. Para ahorar tiempo, únicamente
        lee esos variables que están en la lista de ``ModeloVENSIM.vars_saliendo``.
        """

        for var in símismo.vars_saliendo:
            # Para cada variable que está conectado con el modelo biofísico...

            # Una memoria
            mem_inter = ctypes.create_string_buffer(4)

            # Leer su valor.
            símismo.comanda_vensim(func=símismo.dll.vensim_get_val,
                                   args=[var.encode(), mem_inter],
                                   mensaje_error='Error con VENSIM para leer variables.')

            # Decodar
            val = struct.unpack('f', mem_inter)[0]

            # Guardar en el diccionario interno.
            símismo.variables[var]['val'] = val

    def cerrar_modelo(símismo):
        """
        Cierre la simulación VENSIM.
        """

        # Llamar la comanda para terminar la simulación.
        símismo.comanda_vensim(func=símismo.dll.vensim_command,
                               args="GAME>ENDGAME",
                               mensaje_error='Error para terminar la simulación VENSIM.')

    def verificar_vensim(símismo):
        """
        Esta función regresa el estatus de Vensim. Es particularmente útil para desboguear (no tiene uso en las
        otras funciones de esta clase, y se incluye como ayuda a la programadora.)

        :return: estatus número integral de código de estatus
            | 0 = Vensim está listo
            | 1 = Vensim está en una simulación activa
            | 2 = Vensim está en una simulación, pero no está respondiendo
            | 3 = Malas noticias
            | 4 = Error de memoria
            | 5 = Vensim está en modo de juego
            | 6 = Memoria no libre. Llamar vensim_command() debería de arreglarlo.
            | 16 += ver documentación de VENSIM para vensim_check_status() en la sección de DLL (Suplemento DSS)
        :rtype: int

        """

        # Obtener el estatus.
        estatus = símismo.comanda_vensim(func=símismo.dll.vensim_check_status,
                                         args=[],
                                         mensaje_error='Error verificando el estatus de VENSIM. De verdad, la cosa'
                                                       'te va muy mal.',
                                         val_error=-1, devolver=True)
        return int(estatus)

    @staticmethod
    def comanda_vensim(func, args, mensaje_error=None, val_error=None, devolver=False):
        """
        Esta función sirve para llamar todo tipo de comanda VENSIM.

        :param func: La función DLL a llamar.
        :type func: function

        :param args: Los argumento a pasar a la función. Si no hay, usar una lista vacía.
        :type args: list | str

        :param mensaje_error: El mensaje de error para mostrar si hay un error en la comanda.
        :type mensaje_error: str

        :param val_error: Un valor de regreso VENSIM que indica un error para esta función. Si se deja ``None``, todos
        valores que no son 1 se considerarán como erróneas.

        :type val_error: int

        :param devolver: Si se debe devolver el valor devuelto por VENSIM o no.
        :type devolver: bool

        """

        # Asegurarse que args es una lista
        if type(args) is not list:
            args = [args]

        # Encodar en bytes todos los argumentos de texto.
        for n, a in enumerate(args):
            if type(a) is str:
                args[n] = a.encode()

        # Llamar la función VENSIM y guardar el resultado.
        resultado = func(*args)

        # Verificar su hubo un error.
        if val_error is None:
            error = (resultado != 1)
        else:
            error = (resultado == val_error)

        # Si hubo un error, avisar el usuario.
        if error:
            if mensaje_error is None:
                mensaje_error = 'Error con la comanda VENSIM.'

            mensaje_error += ' Código de error {}.'.format(resultado)

            raise OSError(mensaje_error)

        # Devolver el valor devuelto por la función VENSIM, si aplica.
        if devolver:
            return resultado


def generar_mds(archivo):
    """
    Esta función genera una instancia de modelo de DS. Identifica el tipo de archivo por su extensión (p. ej., .vpm) y
    después genera una instancia de la subclase apropiada de ``~Tinamit.MDS.EnvolturaMDS``.

    :param archivo: El archivo del modelo DS.
    :type archivo: str

    :return: Un modelo DS.
    :rtype: tinamit.MDS.EnvolturaMDS

    """

    # Identificar la extensión.
    ext = os.path.splitext(archivo)[1]

    # Crear la instancia de modelo apropiada para la extensión del archivo.
    if ext == '.vpm':
        # Modelos VENSIM
        return ModeloVENSIM(archivo)
    else:
        # Agregar otros tipos de modelos DS aquí.

        # Mensaje para modelos todavía no incluidos en Tinamit.
        raise ValueError('El tipo de modelo "{}" no se acepta como modelo DS en Tinamit al momento. Si piensas'
                         'que podrías contribuir aquí, ¡contáctenos!'.format(ext))



import os
import tkinter as tk
import pkg_resources


def imagen(nombre):
    archivo = pkg_resources.resource_filename('tinamit.Interfaz.Imágenes', archivos_imgs[nombre])
    img = tk.PhotoImage(file=archivo)
    return img

archivos_imgs = {'LogoInic': 'LogoInic.png',

                 'BtRegrCent_norm': 'BtRegrCent_norm.png',
                 'BtRegrCent_sel': 'BtRegrCent_sel.png',

                 'LogoCent': 'LogoCent.png',
                 'BtLeng_norm': 'BtLeng_norm.png',
                 'BtLeng_sel': 'BtLeng_sel.png',
                 'BtNuevo_norm': 'BtNuevo_norm.png',
                 'BtNuevo_sel': 'BtNuevo_sel.png',
                 'BtGuardar_norm': 'BtGuardar_norm.png',
                 'BtGuardar_sel': 'BtGuardar_sel.png',
                 'BtGuardarComo_norm': 'BtGuardarComo_norm.png',
                 'BtGuardarComo_sel': 'BtGuardarComo_sel.png',
                 'BtAbrir_norm': 'BtAbrir_norm.png',
                 'BtAbrir_sel': 'BtAbrir_sel.png',

                 'BtNavIzq_1_norm': 'BtNavIzq_1_norm.png',
                 'BtNavIzq_1_bloq': 'BtNavIzq_1_bloq.png',
                 'BtNavIzq_1_sel': 'BtNavIzq_1_sel.png',
                 'BtNavIzq_2_norm': 'BtNavIzq_2_norm.png',
                 'BtNavIzq_2_bloq': 'BtNavIzq_2_bloq.png',
                 'BtNavIzq_2_sel': 'BtNavIzq_2_sel.png',
                 'BtNavIzq_3_norm': 'BtNavIzq_3_norm.png',
                 'BtNavIzq_3_bloq': 'BtNavIzq_3_bloq.png',
                 'BtNavIzq_3_sel': 'BtNavIzq_3_sel.png',
                 'BtNavIzq_4_norm': 'BtNavIzq_4_norm.png',
                 'BtNavIzq_4_bloq': 'BtNavIzq_4_bloq.png',
                 'BtNavIzq_4_sel': 'BtNavIzq_4_sel.png',

                 'BtNavEtp_adel_norm': 'BtNavEtp_adel_norm.png',
                 'BtNavEtp_adel_bloq': 'BtNavEtp_adel_bloq.png',
                 'BtNavEtp_adel_sel': 'BtNavEtp_adel_sel.png',
                 'BtNavEtp_atrs_norm': 'BtNavEtp_atrs_norm.png',
                 'BtNavEtp_atrs_bloq': 'BtNavEtp_atrs_bloq.png',
                 'BtNavEtp_atrs_sel': 'BtNavEtp_atrs_sel.png',

                 'BtNavSub_adel_norm': 'BtNavSub_adel_norm.png',
                 'BtNavSub_adel_bloq': 'BtNavSub_adel_bloq.png',
                 'BtNavSub_adel_sel': 'BtNavSub_adel_sel.png',
                 'BtNavSub_atrs_norm': 'BtNavSub_atrs_norm.png',
                 'BtNavSub_atrs_bloq': 'BtNavSub_atrs_bloq.png',
                 'BtNavSub_atrs_sel': 'BtNavSub_atrs_sel.png',


                 'BtBorrarItema_norm': 'BtBorrarItema_norm.png',
                 'BtBorrarItema_sel': 'BtBorrarItema_sel.png',
                 'BtBorrarItema_bloq': 'BtBorrarItema_bloq.png',
                 'BtEditarItema_norm': 'BtEditarItema_norm.png',
                 'BtEditarItema_sel': 'BtEditarItema_sel.png',

                 'BtCasilla_norm': 'BtCasilla_norm.png',
                 'BtCasilla_sel': 'BtCasilla_sel.png',
                 'BtCasilla_bloq': 'BtCasilla_bloq.png',

                 'BtMarca_norm': 'BtMarca_norm.png',
                 'BtMarca_sel': 'BtMarca_sel.png',
                 'BtMarca_bloq': 'BtMarca_bloq.png',

                 'BtPlus_norm': 'BtPlus_norm.png',
                 'BtPlus_sel': 'BtPlus_sel.png',

                 'BtEscribIzqDer': 'BtEscribIzqDer.png',
                 'BtEscribDerIzq': 'BtEscribDerIzq.png',

                 'BtConecciónIzqDer': 'BtConecciónIzqDer.png',
                 'BtConecciónDerIzq': 'BtConecciónDerIzq.png',

                 'FlchConex_colaizq': 'FlchConex_colaizq.png',
                 'FlchConex_colader': 'FlchConex_colader.png',
                 'FlchConex_cbzizq': 'FlchConex_cbzizq.png',
                 'FlchConex_cbzder': 'FlchConex_cbzder.png'

                 }


def escalar_colores(color1, color2, n):

    def escalar(val1, val2, m):
            escl = [val1/256 + (val2/256 - val1/256)*i/(m-1) for i in range(m)]
            return escl

    r = escalar(color1[0], color2[0], n)
    v = escalar(color1[1], color2[1], n)
    a = escalar(color1[2], color2[2], n)

    return list(zip(r, v, a))


def inter_color(colores, p, tipo='rgb'):

    def interpol(val1, val2, u):
        return int(val1 + (val2 - val1)*u)

    def interpol_col(col1, col2, u):
        return tuple([interpol(col1[n], col2[n], u) for n in range(3)])

    pos = (p * (len(colores)-1)) % 1
    inic = max(0, int(pos) - 1)
    fin = inic + 1
    if fin >= len(colores):
        fin = len(colores) - 1
    col = interpol_col(colores[inic], colores[fin], u=pos)

    if tipo == 'rgb':
        col_final = col
    elif tipo == 'hex':
        col_final = '#%02x%02x%02x' % col
    else:
        raise KeyError

    return col_final



import tkinter as tk

try:
    from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
    from matplotlib.figure import Figure
except ImportError:
    pass

from . import Arte as Art
from . import Botones as Bt
from . import Formatos as Fm
from tinamit.Interfaz.Formatos import gen_formato as gf


class ListaItemas(tk.Frame):
    def __init__(símismo, pariente, ubicación, tipo_ubic, encabezado=False, formato_cj=Fm.formato_CjLstItemas):
        super().__init__(pariente, **formato_cj)
        símismo.ancho = ubicación['width']
        símismo.objetos = {}
        símismo.itemas = []

        ubic_tela = gf(Fm.ubic_TlLstItemas.copy())
        if encabezado:
            ubic_tela['y'] = Fm.ubic_EncbzLstItemas['height']
            ubic_tela['height'] = -Fm.ubic_EncbzLstItemas['height']
        símismo.Tela = tk.Canvas(símismo, **Fm.formato_TlLstItemas)
        símismo.Tela.place(**ubic_tela)
        símismo.Caja = tk.Frame(símismo.Tela, **Fm.formato_cajas)
        símismo.BaraDesp = tk.Scrollbar(símismo.Tela, orient="vertical", command=símismo.Tela.yview)
        símismo.Tela.configure(yscrollcommand=símismo.BaraDesp.set)

        símismo.BaraDesp.pack(**gf(Fm.ubic_BaraDesp))

        if Fm.IzqaDerech:
            x = 0
        else:
            x = 20
        símismo.Tela.create_window(x, 1, window=símismo.Caja, tags="self.frame", width=símismo.ancho-20, **Fm.ubic_CjTl)

        símismo.Caja.bind("<Configure>", símismo.ajust_auto)

        if tipo_ubic == 'pack:':
            símismo.pack(**ubicación)
        elif tipo_ubic == 'place':
            símismo.place(**ubicación)

    def ajust_auto(símismo, evento):
        símismo.Tela.configure(scrollregion=símismo.Tela.bbox("all"))

    def añadir(símismo, itema):
        símismo.itemas.append(itema)
        itema.pack(**gf(Fm.ubic_CjItemas))

    def quitar(símismo, itema):
        símismo.itemas.remove(itema)
        if itema.objeto is not None:
            símismo.objetos.pop(itema.objeto.nombre)
        itema.destroy()

    def borrar(símismo):
        for i in símismo.itemas:
            if i.objeto is not None:
                símismo.objetos.pop(i.objeto.nombre)
            i.destroy()
        símismo.itemas = []


class ListaEditable(ListaItemas):
    def __init__(símismo, pariente, ubicación, tipo_ubic):
        super().__init__(pariente, ubicación=ubicación, tipo_ubic=tipo_ubic, encabezado=True)
        símismo.controles = None

    def gen_encbz(símismo, nombres_cols, anchuras):
        cj_encbz = tk.Frame(símismo, **Fm.formato_cajas)

        ancho_bts = Fm.ancho_cj_bts_itemas
        x = [-ancho_bts*n/len(anchuras) for n in range(len(anchuras))]
        relx = [sum(anchuras[:n]) for n in range(len(anchuras))]
        ajust_ancho = [0] * (len(anchuras)-1) + [ancho_bts]
        cols = []
        for n, col in enumerate(nombres_cols):
            ubic = dict(relx=relx[n], x=x[n], **Fm.ubic_ColsEncbzLst)
            cols.append(tk.Label(cj_encbz, text=col, **gf(Fm.formato_EtiqEncbzLst)))
            cols[-1].place(width=ajust_ancho[n], relwidth=anchuras[n], **gf(ubic))

        cj_encbz.place(**gf(Fm.ubic_EncbzLstItemas))

    def editar(símismo, itema):
        símismo.controles.editar(itema)

    def añadir(símismo, itema):
        if itema.objeto is not None:
            símismo.objetos[itema.objeto.nombre] = itema.objeto
        super().añadir(itema)

    def quitar(símismo, itema):
        símismo.controles.borrar()
        super().quitar(itema)


class Itema(tk.Frame):
    def __init__(símismo, lista_itemas, objeto=None, creando_manual=True):
        super().__init__(lista_itemas.Caja, **Fm.formato_cajas)
        símismo.objeto = objeto
        símismo.lista = lista_itemas

        símismo.creando_manual = creando_manual

        símismo.añadir()

    def quitar(símismo):
        símismo.lista.quitar(símismo)

    def añadir(símismo):
        símismo.lista.añadir(símismo)


class ItemaEditable(Itema):
    def __init__(símismo, grupo_control, lista_itemas, creando_manual=True):
        símismo.grupo_control = grupo_control
        if creando_manual:
            símismo.receta = grupo_control.receta

        super().__init__(lista_itemas, grupo_control.objeto, creando_manual=creando_manual)

        símismo.lista_itemas = lista_itemas
        símismo.columnas = []

        símismo.cj_cols = NotImplemented

        símismo.cj_bts = tk.Frame(símismo, **Fm.formato_cajas)

        símismo.bt_editar = Bt.BotónImagen(símismo.cj_bts, comanda=símismo.editar, formato=Fm.formato_botones,
                                           img_norm=Art.imagen('BtEditarItema_norm'),
                                           img_sel=Art.imagen('BtEditarItema_sel'),
                                           ubicación=gf(Fm.ubic_BtsItemas), tipo_ubic='pack')
        símismo.bt_borrar = Bt.BotónImagen(símismo.cj_bts, comanda=símismo.quitar, formato=Fm.formato_botones,
                                           img_norm=Art.imagen('BtBorrarItema_norm'),
                                           img_sel=Art.imagen('BtBorrarItema_sel'),
                                           ubicación=gf(Fm.ubic_BtsItemas), tipo_ubic='pack')

        símismo.bind('<Enter>', lambda event, i=símismo: i.resaltar())
        símismo.bind('<Leave>', lambda event, i=símismo: i.desresaltar())

    def estab_columnas(símismo, anchuras):
        x = [sum(anchuras[:n]) for n in range(len(anchuras))]
        for n, col in enumerate(símismo.columnas):
            ubic = dict(relx=x[n], **Fm.ubic_ColsItemasEdit)
            col.place(relwidth=anchuras[n], **gf(ubic))

        símismo.cj_cols.pack(**gf(Fm.ubic_CjColsItemas))
        símismo.cj_bts.pack(**gf(Fm.ubic_CjBtsItemas))

    def editar(símismo):
        símismo.lista.editar(símismo)

    def actualizar(símismo):
        raise NotImplementedError

    def resaltar(símismo):
        raise NotImplementedError

    def desresaltar(símismo):
        raise NotImplementedError


class GrupoControles(object):
    def __init__(símismo, controles, constructor_itema=None, itema=None, gráfico=None, lista=None,
                 bt_guardar=None, bt_borrar=None, comanda=None):
        símismo.controles = controles
        símismo.constructor_itema = constructor_itema
        símismo.itema = itema
        símismo.gráfico = gráfico
        símismo.lista = lista
        símismo.bt_guardar = bt_guardar
        símismo.bt_borrar = bt_borrar
        símismo.comanda = comanda
        símismo.objeto = None
        símismo.receta = {}

        for ll in símismo.controles:
            símismo.controles[ll].comanda = símismo.cambió_control

        if símismo.lista is not None:
            símismo.lista.controles = símismo

        if símismo.gráfico is not None:
            símismo.gráfico.controles = símismo.controles

        if símismo.bt_guardar is not None:
            símismo.bt_guardar.estab_comanda(símismo.guardar)
            símismo.bt_guardar.bloquear()
        if símismo.bt_borrar is not None:
            símismo.bt_borrar.estab_comanda(símismo.borrar)
            símismo.bt_borrar.bloquear()

    def cambió_control(símismo, *args):
        if símismo.bt_borrar is not None:
            símismo.bt_borrar.desbloquear()
        if símismo.verificar_completo() is True:
            símismo.recrear_objeto()
            if símismo.gráfico is not None:
                símismo.gráfico.objeto = símismo.objeto
                símismo.gráfico.redibujar()
            if símismo.itema is not None:
                símismo.lista.objeto = símismo.objeto
            if símismo.bt_guardar is not None:
                símismo.bt_guardar.desbloquear()
        else:
            if símismo.bt_guardar is not None:
                símismo.bt_guardar.bloquear()

        if símismo.comanda is not None:
            símismo.comanda()

    def guardar(símismo, borrar=True):
        if símismo.itema is not None:
            símismo.itema.actualizar()
        elif símismo.constructor_itema is not None:
            símismo.itema = símismo.constructor_itema(símismo, símismo.lista)

        if borrar:
            símismo.borrar()
            símismo.itema = None

    def borrar(símismo):
        símismo.objeto = None
        símismo.receta = {}
        for ll, control in símismo.controles.items():
            control.borrar()

        if símismo.gráfico is not None:
            símismo.gráfico.borrar()

        if símismo.bt_guardar is not None:
            símismo.bt_guardar.bloquear()
        if símismo.bt_borrar is not None:
            símismo.bt_borrar.bloquear()

        if símismo.comanda is not None:
            símismo.comanda()

    def editar(símismo, itema):
        símismo.borrar()
        símismo.itema = itema
        símismo.objeto = itema.objeto
        for i in itema.receta:
            símismo.controles[i].poner(itema.receta[i])

        símismo.receta = itema.receta

    def verificar_completo(símismo):
        raise NotImplementedError

    def recrear_objeto(símismo):
        raise NotImplementedError


class Gráfico(object):
    def __init__(símismo, pariente, ubicación, tipo_ubic):
        símismo.objeto = None
        símismo.controles = None

        cuadro = Figure()
        cuadro.patch.set_facecolor(Fm.col_fondo)
        símismo.fig = cuadro.add_subplot(111)
        símismo.tela = FigureCanvasTkAgg(cuadro, master=pariente)
        símismo.tela.show()

        if tipo_ubic == 'place':
            símismo.tela.get_tk_widget().place(**ubicación)
        elif tipo_ubic == 'pack':
            símismo.tela.get_tk_widget().pack(**ubicación)

    def redibujar(símismo):
        símismo.borrar()
        símismo.dibujar()
        símismo.tela.draw()

    def borrar(símismo):
        while len(símismo.fig.patches):
            símismo.fig.patches[0].remove()
        while len(símismo.fig.lines):
            símismo.fig.lines[0].remove()
        símismo.tela.draw()

    def dibujar(símismo):
        raise NotImplementedError


class CampoIngreso(object):
    def __init__(símismo, pariente, nombre, val_inic, comanda, ubicación, tipo_ubic, ancho, orden='texto'):
        símismo.comanda = comanda
        símismo.val_inic = val_inic

        símismo.var = tk.StringVar()
        símismo.var.set(símismo.val_inic)
        símismo.var.trace('w', símismo.acción_cambió)

        símismo.val = val_inic

        cj = tk.Frame(pariente, **Fm.formato_cajas)

        símismo.CampoIngr = tk.Entry(cj, textvariable=símismo.var, width=ancho, **Fm.formato_CampoIngr)

        if nombre is not None:
            símismo.Etiq = tk.Label(cj, text=nombre, **Fm.formato_EtiqCtrl)

        if orden is 'texto':
            if nombre is not None:
                símismo.Etiq.pack(**gf(Fm.ubic_EtiqIngrNúm))
            símismo.CampoIngr.pack(**gf(Fm.ubic_CampoIngrEscl))
        else:
            símismo.CampoIngr.pack(**gf(Fm.ubic_CampoIngrEscl))
            if nombre is not None:
                símismo.Etiq.pack(**gf(Fm.ubic_EtiqIngrNúm))

        if tipo_ubic == 'pack':
            cj.pack(**ubicación)
        elif tipo_ubic == 'place':
            cj.place(**ubicación)

    def acción_cambió(símismo, *args):
        nueva_val = símismo.var.get()

        try:
            nueva_val = símismo.validar_ingreso(nueva_val)
            símismo.CampoIngr.config(**Fm.formato_CampoIngr)

            if nueva_val == '':
                símismo.val = None
                return
            if nueva_val != símismo.val:
                símismo.val = nueva_val
                if símismo.comanda is not None:
                    símismo.comanda(nueva_val)

        except ValueError:
            símismo.CampoIngr.config(**Fm.formato_CampoIngr_error)
            return

    def borrar(símismo):
        símismo.var.set(símismo.val_inic)

    def bloquear(símismo):
        símismo.CampoIngr.configure(state=tk.DISABLED, cursor='X_cursor', **Fm.formato_BtMn_bloq)
        símismo.Etiq.config(**Fm.formato_EtiqCtrl_bloq)

    def desbloquear(símismo):
        símismo.CampoIngr.configure(state=tk.NORMAL, cursor='arrow', **Fm.formato_BtMn)
        símismo.Etiq.config(**Fm.formato_EtiqCtrl)

    def poner(símismo, valor):
        símismo.var.set(str(valor))

    def validar_ingreso(símismo, nueva_val):
        raise NotImplementedError


class IngrNúm(CampoIngreso):
    def __init__(símismo, pariente, límites, prec, ubicación, tipo_ubic, lím_incl=True, ancho=5, comanda=None,
                 nombre=None, val_inic='', orden='texto'):

        if prec not in ['dec', 'ent']:
            raise ValueError('"Prec" debe ser uno de "ent" o "dec".')
        símismo.prec = prec

        símismo.val_inic = val_inic
        símismo.lím_incl = lím_incl

        if límites is None:
            límites = (float('-Inf'), float('Inf'))
        if límites[0] is None:
            límites = (float('-Inf'), límites[1])
        if límites[1] is None:
            límites = (límites[0], float('Inf'))
        símismo.límites = límites

        super().__init__(pariente, nombre, val_inic, comanda, ubicación, tipo_ubic, ancho, orden=orden)

    def validar_ingreso(símismo, nueva_val):
        if símismo.prec == 'ent':
            nueva_val = int(nueva_val)
        elif símismo.prec == 'dec':
            nueva_val = round(float(nueva_val), 3)
        else:
            raise KeyError('"Prec" debe ser uno de "ent" o "dec".')

        if símismo.lím_incl:
            if not (símismo.límites[0] <= nueva_val <= símismo.límites[1]):
                raise ValueError
        else:
            if not (símismo.límites[0] < nueva_val < símismo.límites[1]):
                raise ValueError

        return nueva_val


class IngrTexto(CampoIngreso):
    def __init__(símismo, pariente, nombre, ubicación, tipo_ubic, comanda=None):
        super().__init__(pariente, nombre=nombre, val_inic='', comanda=comanda,
                         ubicación=ubicación, tipo_ubic=tipo_ubic, ancho=20)

    def validar_ingreso(símismo, nueva_val):
        return nueva_val


class Menú(object):
    def __init__(símismo, pariente, opciones, ubicación, tipo_ubic,
                 nombre=None, comanda=None, texto_opciones=None, inicial='',
                 formato_bt=Fm.formato_BtMn, formato_mn=Fm.formato_MnMn, ancho=15):
        símismo.opciones = opciones
        if texto_opciones is None:
            texto_opciones = opciones
        símismo.conv = {}

        símismo.comanda = comanda

        símismo.val_inicial = inicial
        símismo.var = tk.StringVar()

        símismo.val = inicial
        símismo.exclusivos = []

        cj = tk.Frame(pariente, **Fm.formato_cajas)

        if nombre is not None:
            símismo.Etiq = tk.Label(cj, text=nombre, **Fm.formato_EtiqCtrl)
            símismo.Etiq.pack(**gf(Fm.ubic_EtiqMenú))

        símismo.MenúOpciones = tk.OptionMenu(cj, símismo.var, '')
        símismo.MenúOpciones.config(takefocus=True)
        símismo.refrescar(opciones, texto_opciones)
        símismo.var.trace('w', símismo.acción_cambió)

        símismo.MenúOpciones.config(width=ancho, **formato_bt)
        símismo.MenúOpciones['menu'].config(**formato_mn)

        símismo.MenúOpciones.pack(**gf(Fm.ubic_Menú))

        if tipo_ubic == 'pack':
            cj.pack(**ubicación)
        elif tipo_ubic == 'place':
            cj.place(**ubicación)

    def estab_exclusivo(símismo, otro_menú):
        assert type(otro_menú) is Menú
        if otro_menú not in símismo.exclusivos:
            símismo.exclusivos.append(otro_menú)
        if símismo not in otro_menú.exclusivos:
            otro_menú.estab_exclusivo(símismo)

    def refrescar(símismo, opciones, texto_opciones=None, alfabetizar=True):

        símismo.opciones = opciones
        if texto_opciones is None:
            texto_opciones = opciones

        símismo.MenúOpciones['menu'].delete(0, 'end')

        símismo.conv = {'': ''}

        if alfabetizar:
            lista = sorted(zip(texto_opciones, opciones))
        else:
            lista = zip(texto_opciones, opciones)

        for tx, op in lista:
            símismo.conv[op] = tx
            símismo.MenúOpciones['menu'].add_command(label=tx, command=tk._setit(símismo.var, tx))

        símismo.borrar()

    def acción_cambió(símismo, *args):
        texto_campo = símismo.var.get()
        nueva_val = [ll for ll, v in símismo.conv.items() if v == texto_campo][0]

        if nueva_val != símismo.val:
            if nueva_val == '':
                nueva_val = None
            else:
                for menú in símismo.exclusivos:
                    menú.excluir(nueva_val)
                    if símismo.val is not None:
                        menú.reinstaurar(símismo.val)

            símismo.val = nueva_val
            if símismo.comanda is not None:
                símismo.comanda(nueva_val)

    def excluir(símismo, valor):
        menú = símismo.MenúOpciones['menu']
        i = símismo.opciones.index(valor)
        menú.entryconfig(i, state=tk.DISABLED)

    def reinstaurar(símismo, valor):
        menú = símismo.MenúOpciones['menu']
        i = símismo.opciones.index(valor)
        menú.entryconfig(i, state=tk.NORMAL)

    def borrar(símismo):
        símismo.val = símismo.val_inicial
        símismo.var.set(símismo.conv[símismo.val])

    def bloquear(símismo):
        símismo.MenúOpciones.configure(state=tk.DISABLED, cursor='X_cursor', **Fm.formato_BtMn_bloq)
        símismo.Etiq.config(**Fm.formato_EtiqCtrl_bloq)

    def desbloquear(símismo):
        símismo.MenúOpciones.configure(state=tk.NORMAL, cursor='arrow', **Fm.formato_BtMn)
        símismo.Etiq.config(**Fm.formato_EtiqCtrl)

    def poner(símismo, valor):
        símismo.var.set(símismo.conv[valor])
        símismo.val = valor


class Escala(object):
    def __init__(símismo, pariente, texto, límites, prec, ubicación, tipo_ubic, comanda=None, valor_inicial=None):
        símismo.cj = tk.Frame(pariente, **Fm.formato_cajas)
        símismo.comanda = comanda
        if valor_inicial is None:
            valor_inicial = (límites[0] + límites[1])/2
        símismo.valor_inicial = valor_inicial

        símismo.val = símismo.valor_inicial

        símismo.etiq = tk.Label(símismo.cj, text=texto, **Fm.formato_EtiqCtrl)

        etiq_inic = tk.Label(símismo.cj, text=límites[0], **Fm.formato_EtiqNúmEscl)
        símismo.escl = CosoEscala(símismo, límites, val_inic=valor_inicial, prec=prec)
        etiq_fin = tk.Label(símismo.cj, text=límites[1], **Fm.formato_EtiqNúmEscl)

        símismo.etiq.pack(**gf(Fm.ubic_EtiqEscl))
        etiq_inic.pack(**gf(Fm.ubic_EtiqNúmEscl))
        símismo.escl.pack(**gf(Fm.ubic_Escl))
        etiq_fin.pack(**gf(Fm.ubic_EtiqNúmEscl))

        símismo.ingr = IngrNúm(símismo.cj, nombre=None, límites=límites, val_inic=valor_inicial, prec=prec,
                               ubicación=gf(Fm.ubic_CampoIngrEscl), tipo_ubic='pack',
                               comanda=símismo.cambio_ingr)

        if tipo_ubic == 'pack':
            símismo.cj.pack(**ubicación)
        elif tipo_ubic == 'place':
            símismo.cj.place(**ubicación)

    def cambio_ingr(símismo, val):
        if val != símismo.escl.val:
            símismo.escl.poner(float(val))
            símismo.val = float(val)
            if símismo.comanda is not None:
                símismo.comanda(val)

    def cambio_escl(símismo, val):
        if str(val) != símismo.val:
            símismo.val = val
            símismo.ingr.var.set(val)

            if símismo.comanda is not None:
                símismo.comanda(val)

    def borrar(símismo):
        pass
        # símismo.escl.poner(símismo.valor_inicial)
        # símismo.ingr.var.set(símismo.valor_inicial)
        # símismo.val = símismo.valor_inicial

    def poner(símismo, valor):
        símismo.escl.poner(valor)
        símismo.ingr.var.set(valor)
        símismo.val = valor


class CosoEscala(tk.Canvas):
    def __init__(símismo, pariente, límites, val_inic, ancho=150, altura=30, prec='cont'):
        super().__init__(pariente.cj, width=ancho, height=altura, background=Fm.col_fondo, highlightthickness=0)

        símismo.dim = (ancho, altura)
        símismo.límites = límites
        símismo.pariente = pariente
        símismo.tipo = prec
        símismo.val = val_inic

        símismo.create_rectangle(0, 0, 1, altura, **Fm.formato_lín_escl)
        símismo.create_rectangle(ancho-2, 0, ancho, altura, **Fm.formato_lín_escl)
        símismo.create_line(0, round(altura/2), ancho, round(altura/2), fill=Fm.col_1)

        símismo.info_mov = {"x": 0, "y": 0}

        símismo.manilla = símismo.create_rectangle(float(ancho / 2) - 3, 0, float(ancho / 2) + 3, altura,
                                                   outline=Fm.col_5, fill=Fm.col_5)

        símismo.tag_bind(símismo.manilla, "<ButtonPress-1>", símismo.acción_empujar)
        símismo.tag_bind(símismo.manilla, "<ButtonRelease-1>", símismo.acción_soltar)
        símismo.tag_bind(símismo.manilla, "<B1-Motion>", símismo.acción_movimiento)
        símismo.bind("<ButtonRelease-1>", símismo.acción_soltar)

        símismo.poner(símismo.val)

    def acción_empujar(símismo, event):
        símismo.info_mov["x"] = (símismo.coords(símismo.manilla)[0] + símismo.coords(símismo.manilla)[2])/2

    def acción_soltar(símismo, event):
        símismo.info_mov["x"] = 0

    def acción_movimiento(símismo, event):
        x = event.x
        if x > símismo.dim[0]:
            x = símismo.dim[0]
        elif x < 0:
            x = 0

        símismo.val = round(x/símismo.dim[0] * (símismo.límites[1] - símismo.límites[0]) + símismo.límites[0], 3)

        if símismo.tipo == 'ent':
            símismo.val = int(símismo.val)
            x = (símismo.val - símismo.límites[0])/(símismo.límites[1] - símismo.límites[0])*símismo.dim[0]

        delta_x = x - símismo.info_mov["x"]

        símismo.move(símismo.manilla, delta_x, 0)

        símismo.info_mov["x"] = x

        símismo.pariente.cambio_escl(símismo.val)

    def poner(símismo, val):
        símismo.val = val
        x_manilla = (símismo.coords(símismo.manilla)[0] + símismo.coords(símismo.manilla)[2])/2
        nuevo_x = (símismo.val - símismo.límites[0])/(símismo.límites[1] - símismo.límites[0])*símismo.dim[0]
        delta_x = nuevo_x - x_manilla
        símismo.move(símismo.manilla, delta_x, 0)



import tkinter as tk

from . import Arte as Art
from . import Botones as Bt
from . import ControlesGenéricos as CtrG
from . import Formatos as Fm
from .Formatos import gen_formato as gf


# Caja lenguas
class CajaAñadirLeng(tk.Frame):
    def __init__(símismo, pariente, caja):
        super().__init__(caja, **Fm.formato_cajas)
        Bt.BotónImagen(símismo, comanda=símismo.acción_bt_plus, formato=Fm.formato_botones,
                       img_norm=Art.imagen('BtPlus_norm'), img_sel=Art.imagen('BtPlus_sel'),
                       ubicación=gf(Fm.ubic_CjCamposAñLeng), tipo_ubic='pack')
        símismo.pariente = pariente

        símismo.cj_campos = cj_campos = tk.Frame(símismo, **Fm.formato_cajas)
        símismo.ingr_leng = CtrG.IngrTexto(cj_campos, nombre=None,
                                           ubicación=gf(Fm.ubic_CjCamposAñLeng), tipo_ubic='pack')

        Bt.BotónImagen(cj_campos, comanda=símismo.acción_bt_guardar, formato=Fm.formato_botones,
                       img_norm=Art.imagen('BtMarca_norm'), img_sel=Art.imagen('BtMarca_sel'),
                       ubicación=gf(Fm.ubic_BtsAñadirLeng), tipo_ubic='pack')

        símismo.IzqDer = Bt.BotónAltern(cj_campos, formato=Fm.formato_botones,
                                        img_1=Art.imagen('BtEscribIzqDer'), img_2=Art.imagen('BtEscribDerIzq'),
                                        ubicación=gf(Fm.ubic_BtsAñadirLeng), tipo_ubic='pack')

        símismo.mostrando = False

    def acción_bt_plus(símismo):
        if not símismo.mostrando:
            símismo.cj_campos.pack(**gf(Fm.ubic_CjCamposAñLeng))
        else:
            símismo.cj_campos.pack_forget()

        símismo.mostrando = not símismo.mostrando

    def acción_bt_guardar(símismo):
        nombre = símismo.ingr_leng.val
        if nombre is not None and len(nombre):
            izqder = símismo.IzqDer.val
            símismo.pariente.añadir_lengua(nombre=nombre, izqder=izqder)
        símismo.ingr_leng.borrar()


class ItemaLeng(CtrG.Itema):
    def __init__(símismo, pariente, lista, nombre, lengua, estado, utilizando=False, utilizable=True, borrable=True):
        super().__init__(lista_itemas=lista)
        símismo.nombre = nombre
        símismo.lengua = lengua

        cj_bt_utilizar = tk.Frame(símismo, **Fm.formato_secciones_itemas)
        cj_central = tk.Frame(símismo, **Fm.formato_secciones_itemas)
        cj_bts = tk.Frame(símismo, **Fm.formato_secciones_itemas)

        if utilizando:
            img_norm = Art.imagen('BtCasilla_sel')
        else:
            img_norm = Art.imagen('BtCasilla_norm')
        bt_utilizar = Bt.BotónImagen(cj_bt_utilizar, comanda=lambda x=nombre: pariente.utilizar(x),
                                     formato=Fm.formato_botones,
                                     img_norm=img_norm,
                                     img_sel=Art.imagen('BtCasilla_sel'),
                                     img_bloq=Art.imagen('BtCasilla_bloq'),
                                     ubicación=gf(Fm.ubic_IzqLstLeng), tipo_ubic='pack')
        if not utilizable:
            bt_utilizar.bloquear()

        if 0 < estado < 1:
            ancho = Fm.ancho_etiq_nombre_estr
        else:
            ancho = Fm.ancho_etiq_nombre_largo

        cj_etiq_nombre_leng = tk.Frame(cj_central, width=ancho, **Fm.formato_cj_etiq_nombre_leng)
        cj_etiq_nombre_leng.pack_propagate(0)
        símismo.etiq_nombre = tk.Label(cj_etiq_nombre_leng, text=símismo.nombre,
                                       font=Fm.fuente_etiq_itema_norm, **Fm.formato_etiq_nombre_leng)
        símismo.etiq_nombre.pack(**gf(Fm.ubic_etiq_nombre_leng))
        cj_etiq_nombre_leng.pack(**gf(Fm.ubic_EtiqNombreLstLeng))

        if 0 < estado < 1:
            color_barra = Art.inter_color(Fm.colores_prog_leng, p=estado, tipo='hex')
            altura, ancho = Fm.dim_barra_prog_leng
            barra_prog = tk.Canvas(cj_central, width=ancho, height=altura, background=Fm.col_fondo,
                                   highlightthickness=1, highlightbackground=color_barra)
            barra_prog.create_rectangle(0, 0, round(ancho*estado), altura, fill=color_barra, outline=color_barra)
            barra_prog.pack(gf(Fm.ubic_DerLstLeng))

        símismo.bt_editar = Bt.BotónImagen(cj_bts, comanda=lambda x=nombre: pariente.editar(x),
                                           formato=Fm.formato_botones,
                                           img_norm=Art.imagen('BtEditarItema_norm'),
                                           img_sel=Art.imagen('BtEditarItema_sel'),
                                           ubicación=gf(Fm.ubic_IzqLstLeng), tipo_ubic='pack')
        símismo.bt_borrar = Bt.BotónImagen(cj_bts, comanda=lambda x=nombre: pariente.confirmar_borrar(x),
                                           formato=Fm.formato_botones,
                                           img_norm=Art.imagen('BtBorrarItema_norm'),
                                           img_sel=Art.imagen('BtBorrarItema_sel'),
                                           img_bloq=Art.imagen('BtBorrarItema_bloq'),
                                           ubicación=gf(Fm.ubic_IzqLstLeng), tipo_ubic='pack')
        if not borrable:
            símismo.bt_borrar.bloquear()

        cj_bt_utilizar.pack(**gf(Fm.ubic_IzqLstLeng))
        cj_central.pack(**gf(Fm.ubic_CjCentLstLeng))
        cj_bts.pack(**gf(Fm.ubic_BtsAñadirLeng))


class CajaEditLeng(tk.Frame):
    def __init__(símismo, pariente, apli, leng_base, dic_leng_base, leng_edit, dic_leng_edit):
        super().__init__(pariente, **Fm.formato_CjEditLeng)
        símismo.pariente = pariente
        símismo.apli = apli
        símismo.dic_leng_edit = dic_leng_edit

        cj_cbz = tk.Frame(símismo, **Fm.formato_cajas)

        etiq_cbz = tk.Label(cj_cbz, text=apli.Trads['Traducciones'], **Fm.formato_EtiqCbzEditLeng)

        cj_nombres_lengs = tk.Frame(cj_cbz, **Fm.formato_cajas)
        etiq_base = tk.Label(cj_nombres_lengs, text=leng_base, **Fm.formato_EtiqLengBase)
        etiq_edit = tk.Label(cj_nombres_lengs, text=leng_edit, **Fm.formato_EtiqLengEdit)

        etiq_base.pack(**gf(Fm.ubic_EtiqLengs))
        etiq_edit.pack(**gf(Fm.ubic_EtiqLengs))

        lín_hor_1 = tk.Frame(símismo, **Fm.formato_LínHor)

        etiq_cbz.pack(**gf(Fm.ubic_CjNomEditLeng))
        cj_nombres_lengs.pack(**gf(Fm.ubic_CjNomEditLeng))

        cj_lista = tk.Frame(símismo, **Fm.formato_cajas)
        lista = CtrG.ListaItemas(cj_lista, formato_cj=Fm.formato_LstEditLeng,
                                 ubicación=gf(Fm.ubic_LstEditLeng), tipo_ubic='place')
        símismo.itemas = []
        for ll, texto in sorted(dic_leng_base['Trads'].items()):
            if ll in dic_leng_edit['Trads'].keys():
                texto_trad = dic_leng_edit['Trads'][ll]
            else:
                texto_trad = ''
            nuevo_itema = ItemaEditTrad(lista=lista, llave=ll, texto_origin=texto, texto_trad=texto_trad)
            símismo.itemas.append(nuevo_itema)

        lín_hor_2 = tk.Frame(símismo, **Fm.formato_LínHor)

        cj_bts = tk.Frame(símismo, **Fm.formato_cajas)
        símismo.bt_guardar = Bt.BotónTexto(cj_bts, texto=apli.Trads['Guardar'],
                                           formato_norm=Fm.formato_BtGuardar_norm,
                                           formato_sel=Fm.formato_BtGuardar_sel,
                                           ubicación=gf(Fm.ubic_BtsGrupo), tipo_ubic='pack',
                                           comanda=símismo.guardar)

        símismo.bt_no_guardar = Bt.BotónTexto(cj_bts, texto=apli.Trads['NoGuardar'],
                                              formato_norm=Fm.formato_BtBorrar_norm,
                                              formato_sel=Fm.formato_BtBorrar_sel,
                                              ubicación=gf(Fm.ubic_BtsGrupo), tipo_ubic='pack',
                                              comanda=símismo.cerrar)

        cj_cbz.pack(**gf(Fm.ubic_CjCbzCjEditLeng))
        lín_hor_1.pack(**gf(Fm.ubic_LínHorCjEditLeng))
        cj_lista.pack(**gf(Fm.ubic_CjCentrCjEditLeng))
        lín_hor_2.pack(**gf(Fm.ubic_LínHorCjEditLeng))
        cj_bts.pack(**gf(Fm.ubic_CjBtsCjEditLeng))
        símismo.place(**gf(Fm.ubic_CjEditLeng))

    def guardar(símismo):
        for itema in símismo.itemas:
            símismo.dic_leng_edit['Trads'][itema.llave] = itema.sacar_trad()
        símismo.pariente.DicLeng.guardar()
        símismo.pariente.refrescar()
        símismo.cerrar()

    def cerrar(símismo):
        símismo.destroy()


class ItemaEditTrad(CtrG.Itema):
    def __init__(símismo, lista, llave, texto_origin, texto_trad):
        super().__init__(lista_itemas=lista)
        símismo.llave = llave
        cj_tx_orig = tk.Frame(símismo, **Fm.formato_CjLengTxOrig)
        etiq_tx_orig = tk.Label(cj_tx_orig, text=texto_origin, **Fm.formato_EtiqLengTxOrig)
        etiq_tx_orig.pack(side='top')

        cj_trad = tk.Frame(símismo, **Fm.formato_cajas)
        símismo.campo_texto = tk.Text(cj_trad, **Fm.formato_CampoTexto)
        símismo.campo_texto.insert('end', texto_trad)

        símismo.campo_texto.pack()
        cj_tx_orig.pack(**gf(Fm.ubic_CjTxOrigTradLeng))

        cj_trad.pack(**gf(Fm.ubic_EtiqLengs))

    def sacar_trad(símismo):
        return símismo.campo_texto.get('1.0', 'end').replace('\n', '')


class CajaAviso(tk.Frame):
    def __init__(símismo, apli, texto):
        super().__init__(**Fm.formato_CajaAvisoReinic)

        etiq = tk.Label(símismo, text=texto, **Fm.formato_EtiqLengReinic)
        etiq.pack(**gf(Fm.ubic_etiq_aviso_inic_leng))

        símismo.bt = Bt.BotónTexto(símismo, texto=apli.Trads['Entendido'],
                                   formato_norm=Fm.formato_BtAvisoInic_norm,
                                   formato_sel=Fm.formato_BtAvisoInic_sel,
                                   ubicación=gf(Fm.ubic_bt_aviso_inic_leng), tipo_ubic='pack',
                                   comanda=símismo.destroy)

        símismo.place(**gf(Fm.ubic_CjAvisoReinic))


class CajaAvisoBorrar(tk.Frame):
    def __init__(símismo, apli, nombre, acción):
        super().__init__(**Fm.formato_CajaAvisoReinic)
        símismo.acción = acción
        símismo.nombre = nombre

        etiq = tk.Label(símismo, text=apli.Trads["AvisoBorrarLeng"] % nombre, **Fm.formato_EtiqLengBorrar)
        etiq.pack(**gf(Fm.ubic_etiq_aviso_borrar_leng))

        cj_bts = tk.Frame(símismo, **Fm.formato_cajas)
        símismo.bt_no = Bt.BotónTexto(cj_bts, texto=apli.Trads['No'],
                                      formato_norm=Fm.formato_BtGuardar_norm,
                                      formato_sel=Fm.formato_BtGuardar_sel,
                                      ubicación=gf(Fm.ubic_bts_aviso_borrar_leng), tipo_ubic='pack',
                                      comanda=símismo.destroy)

        símismo.bt_sí = Bt.BotónTexto(cj_bts, texto=apli.Trads['Sí'],
                                      formato_norm=Fm.formato_BtBorrar_norm,
                                      formato_sel=Fm.formato_BtBorrar_sel,
                                      ubicación=gf(Fm.ubic_bts_aviso_borrar_leng), tipo_ubic='pack',
                                      comanda=símismo.acción_borrar)

        cj_bts.pack(**gf(Fm.ubic_cj_bts_aviso_borrar_leng))

        símismo.place(**gf(Fm.ubic_CjAvisoReinic))

    def acción_borrar(símismo):
        símismo.acción(símismo.nombre)
        símismo.destroy()


# Subcaja 2.1
class GrpCtrlsConex(CtrG.GrupoControles):
    def __init__(símismo, pariente, apli, controles, lista, bt_guardar, bt_borrar, comanda):
        super().__init__(controles, constructor_itema=ItemaConexión, lista=lista,
                         bt_guardar=bt_guardar, bt_borrar=bt_borrar, comanda=comanda)
        símismo.apli = apli
        símismo.pariente = pariente

    def verificar_completo(símismo):
        campos_necesarios = ['var_mds', 'mds_fuente', 'conv', 'var_bf']
        completos = [símismo.controles[x].val is not None and símismo.controles[x].val is not ''
                     for x in campos_necesarios]
        if min(completos):
            return True
        else:
            return False

    def recrear_objeto(símismo):
        for ll, control in símismo.controles.items():
            símismo.receta[ll] = control.val


class ListaConexiónes(CtrG.ListaEditable):
    def __init__(símismo, pariente, apli, ubicación, tipo_ubic):
        super().__init__(pariente, ubicación=ubicación, tipo_ubic=tipo_ubic)

        símismo.pariente = pariente
        nombres_cols = [apli.Trads['VariableMDS'], '', apli.Trads['VariableBiofísico']]
        anchuras = Fm.anchos_cols_listacon
        símismo.gen_encbz(nombres_cols, anchuras)

    def añadir(símismo, itema):
        super().añadir(itema)
        símismo.pariente.verificar_completo()

    def quitar(símismo, itema):
        super().quitar(itema)
        símismo.pariente.verificar_completo()


class ItemaConexión(CtrG.ItemaEditable):
    def __init__(símismo, grupo_control, lista_itemas, receta=None, creando_manual=True):
        símismo.pariente = grupo_control.pariente
        if receta is not None:
            símismo.receta = receta

        super().__init__(grupo_control=grupo_control, lista_itemas=lista_itemas, creando_manual=creando_manual)

        símismo.cj_cols = cj_cols = tk.Frame(símismo, **Fm.formato_cajas)
        cj_var_mds = tk.Frame(cj_cols, **Fm.formato_secciones_itemas)
        cj_conv = tk.Frame(cj_cols, **Fm.formato_secciones_itemas)
        cj_var_bf = tk.Frame(cj_cols, **Fm.formato_secciones_itemas)

        símismo.flecha = {'cola_izq': Art.imagen('FlchConex_colaizq'),
                          'cola_der': Art.imagen('FlchConex_colader'),
                          'cbz_izq': Art.imagen('FlchConex_cbzizq'),
                          'cbz_der': Art.imagen('FlchConex_cbzder')}

        símismo.etiq_varMDS = tk.Label(cj_var_mds, **Fm.formato_texto_itemas)
        símismo.etiq_izqflecha = tk.Label(cj_conv, **Fm.formato_etiq)
        símismo.etiq_conversión = tk.Label(cj_conv, **Fm.formato_etiq_conversión)
        símismo.etiq_derflecha = tk.Label(cj_conv, **Fm.formato_etiq)
        símismo.etiq_varBf = tk.Label(cj_var_bf, **Fm.formato_texto_itemas)

        símismo.etiquetas = [símismo.etiq_varMDS, símismo.etiq_izqflecha, símismo.etiq_conversión,
                             símismo.etiq_derflecha, símismo.etiq_varBf]
        símismo.columnas = [cj_var_mds, cj_conv, cj_var_bf]
        for etiq in símismo.etiquetas:
            etiq.pack(**gf(Fm.ubic_EtiqItemas))

        símismo.estab_columnas(anchuras=Fm.anchos_cols_listacon)

        símismo.actualizar()

    def actualizar(símismo):
        try:
            símismo.etiq_varMDS.config(text=símismo.receta['vars']['mds'])
            símismo.etiq_varBf.config(text=símismo.receta['vars']['bf'])
            modelo_fuente = símismo.receta['modelo_fuente']
        except KeyError:
            símismo.etiq_varMDS.config(text=símismo.receta['var_mds'])
            símismo.etiq_varBf.config(text=símismo.receta['var_bf'])
            if símismo.receta['mds_fuente']:
                modelo_fuente = 'mds'
            else:
                modelo_fuente = 'bf'

        símismo.etiq_conversión.config(text='X %s' % símismo.receta['conv'])

        if modelo_fuente == 'mds':
            if Fm.IzqaDerech:
                símismo.etiq_izqflecha.config(image=símismo.flecha['cola_izq'])
                símismo.etiq_derflecha.config(image=símismo.flecha['cbz_der'])
            else:
                símismo.etiq_izqflecha.config(image=símismo.flecha['cola_der'])
                símismo.etiq_derflecha.config(image=símismo.flecha['cbz_izq'])
        else:
            if Fm.IzqaDerech:
                símismo.etiq_izqflecha.config(image=símismo.flecha['cbz_izq'])
                símismo.etiq_derflecha.config(image=símismo.flecha['cola_der'])
            else:
                símismo.etiq_izqflecha.config(image=símismo.flecha['cbz_der'])
                símismo.etiq_derflecha.config(image=símismo.flecha['cola_izq'])

    def resaltar(símismo):
        for etiq in símismo.etiquetas:
            if etiq is not símismo.etiq_conversión:
                etiq.config(font=gf(Fm.fuente_etiq_itema_sel))

    def desresaltar(símismo):
        for etiq in símismo.etiquetas.copy():
            if etiq is not símismo.etiq_conversión:
                etiq.config(font=gf(Fm.fuente_etiq_itema_norm))

    def añadir(símismo):
        símismo.pariente.añadir_conexión(símismo.receta)
        super().añadir()

    def quitar(símismo):
        símismo.pariente.quitar_conexión(símismo.receta)
        super().quitar()



import ctypes
import random
import struct


# Un ejemplo de base de como conectar e intercambiar valores con Vensim. Este se desarrolló como prueba y NO es
# necesario para el funcionamiento del programa Tinamit.


class Dll(object):
    def __init__(símismo, ubic_modelo):
        símismo.variables = None

        símismo.dll = ctypes.WinDLL('C:\\Windows\\System32\\vendll32.dll')
        resultado = símismo.dll.vensim_command(('SPECIAL>LOADMODEL|"%s"' % ubic_modelo).encode())
        print('Cargando... ', resultado)
        símismo.verificar_estatus()

    def silenciar(símismo):
        símismo.dll.vensim_be_quiet(2)

    def sacar_nombres_vars(símismo):
        mem_inter = ctypes.create_string_buffer(100)
        resultado = símismo.dll.vensim_get_varnames(b"*", 0, mem_inter, 100)
        símismo.variables = [x for x in mem_inter.raw.decode().split('\x00') if x]

        print('Sacando variables... ', resultado)
        print(símismo.variables)
        símismo.verificar_estatus()

    def estab_nombre_corrida(símismo, nombre):
        resultado = símismo.dll.vensim_command(("SIMULATE>RUNNAME|%s" % nombre).encode())
        print('Estableciendo nombre de corrida... ', resultado)
        símismo.verificar_estatus()

    def empezar_simul(símismo):
        resultado = símismo.dll.vensim_start_simulation(1, 0, 1)
        print('Empezando simulación... ', resultado)
        símismo.verificar_estatus()

    def avanzar_simul(símismo, paso):
        resultado = símismo.dll.vensim_continue_simulation(paso)
        print('Avanzando simulación... ', resultado)
        símismo.verificar_estatus()

    def empezar_juego(símismo):
        resultado = símismo.dll.vensim_command(b"MENU>GAME")
        print('Empezando juego... ', resultado)
        símismo.verificar_estatus()

    def estab_paso_juego(símismo, paso):
        resultado = símismo.dll.vensim_command(("GAME>GAMEINTERVAL|%i" % paso).encode())
        print('Estableciendo paso juego... ', resultado)
        símismo.verificar_estatus()

    def avanzar_juego(símismo):
        resultado = símismo.dll.vensim_command(b"GAME>GAMEON")
        print('Avanzando juego... ', resultado)
        símismo.verificar_estatus()

    def terminar_juego(símismo):
        resultado = símismo.dll.vensim_command(b"GAME>ENDGAME")
        print('Terminando juego... ', resultado)
        símismo.verificar_estatus()

    def sacar_val_var(símismo, var, imprim=True):
        mem_inter = ctypes.create_string_buffer(4)
        resultado = símismo.dll.vensim_get_val(var.encode(), mem_inter)
        if imprim:
            print('Sacando valor var %s... ' % var, resultado)
            print(var, struct.unpack('f', mem_inter))
            símismo.verificar_estatus()
        else:
            return float(struct.unpack('f', mem_inter)[0])

    def poner_val_var(símismo, val, var):
        resultado = símismo.dll.vensim_command(('SIMULATE>SETVAL|%s = %f' % (var, val)).encode())
        print('Poniendo valor %f a var %s... ' % (val, var), resultado)
        nuevo_val = símismo.sacar_val_var(var, imprim=False)
        print('Ahora var %s es igual a %f' % (var, nuevo_val))
        símismo.verificar_estatus()

    def terminar_simulación(símismo):
        resultado = símismo.dll.vensim_finish_simulation()
        print('Terminando simulación... ', resultado)
        símismo.verificar_estatus()

    def verificar_estatus(símismo):
        estatus = int(símismo.dll.vensim_check_status())
        if estatus == 0:
            print('Listo.', estatus)
        elif estatus == 1:
            print('Simulando.', estatus)
        elif estatus == 5:
            print('Jugando.', estatus)
        elif estatus == 6:
            print('Memoria no libre.', estatus)
        else:
            print('Tienes un problema.', estatus)

ubicación_modelo = "C:\\Users\\jmalar1\\Documents\\PycharmProjects\\Tinamit\\Prueba dll.vpm"
modelo = Dll(ubicación_modelo)
# modelo.silenciar()
modelo.sacar_nombres_vars()

modelo.estab_nombre_corrida('Corrida')
modelo.sacar_val_var('FINAL TIME')
modelo.poner_val_var(120, 'FINAL TIME')  # Absolutamente necesario


modelo.empezar_juego()

modelo.estab_paso_juego(10)
modelo.sacar_val_var('Time')
modelo.sacar_val_var('Variable 1')

modelo.sacar_val_var('Time')
modelo.sacar_val_var('Nivel')

for i in range(12):
    val_ant = modelo.sacar_val_var('Variable 1', imprim=False)
    val_ahora = val_ant + random.random() - 0.5
    modelo.poner_val_var(val_ahora, 'Variable 1')
    modelo.avanzar_juego()
    modelo.sacar_val_var('Time')
    modelo.sacar_val_var('Nivel')
    input('Presione "Intro" para seguir...')

modelo.terminar_juego()


print('fin')
